%%%%%%%%%%%%%%% Matlab script for generating test instances %%%%%%%%%%%%%%
%                                                                        %
% This script generate test instances to use in the                      %
% MIP-models and the 2DRSSP heuristic.                                   %
%                                                                        %
% Grid layout and exit squares are manually generated                                      %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Choose the amount of squares in the grid
i = input('Add number of rows: ');
j = input('Add number of columns: ');

%Reading information from Excel file "cargo_input_matlab"
%Cargo information variables
C_type = xlsread('cargo_input_matlab.xlsx','B2:B59'); %Cargo type
C_lp = xlsread('cargo_input_matlab.xlsx','C2:C59'); %Loading port
C_up = xlsread('cargo_input_matlab.xlsx','D2:D59'); %Unloading port
C_vehnum = xlsread('cargo_input_matlab.xlsx','E2:E59'); %Total num of vehicle in cargo
C_l = xlsread('cargo_input_matlab.xlsx','F2:F59');  %Vehicle length
C_wi = xlsread('cargo_input_matlab.xlsx','G2:G59'); %Vehicle width
C_we = xlsread('cargo_input_matlab.xlsx','I2:I59'); %Vehicle weight
Rev = xlsread('cargo_input_matlab.xlsx','K2:K59'); %Revenue optional cargo

%Ship information variables
S_l = xlsread('cargo_input_matlab.xlsx','L2:L2'); %Ship length
S_w = xlsread('cargo_input_matlab.xlsx','M2:M2'); %Ship width
S_b = 0.15; %Clearance between cars

%Calculate number of squares each vehicle will take
Square_l = ((C_l + S_b)*i)/S_l; 
Square_w = ((C_wi + S_b)*j)/S_w;

%Number of mandatory and optional cargoes
CargoM = nnz(C_type==1);
CargoO = nnz(C_type==2);

%Create input file for Xpress
testinstance = input('Filename for test instance: ','s');
fileID = fopen(testinstance,'w');

fprintf(fileID,'nRows\t\t:\t%d\r\n',i);
fprintf(fileID,'nColumns\t:\t%d\r\n',j);
fprintf(fileID,'nCargoesM\t:\t%d\r\n',CargoM);
fprintf(fileID,'nCargoesO\t:\t%d\r\n',CargoO);
fprintf(fileID,'nPorts\t\t:\t%d\r\n',max(C_up));
fprintf(fileID,'\r\n');

fprintf(fileID,'SquaresL\t: [\r\n');
fprintf(fileID,'%d\t',ceil(Square_l));
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'SquaresW\t: [\r\n');
fprintf(fileID,'%d\t',ceil(Square_w));
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'LoadingPort\t: [\r\n');
fprintf(fileID,'%d\t',C_lp);
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'UnloadingPort\t: [\r\n');
fprintf(fileID,'%d\t',C_up);
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'NumberVehicle\t: [\r\n');
fprintf(fileID,'%d\t',C_vehnum);
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'CargoWeight\t: [\r\n');
fprintf(fileID,'%d\t',C_we);
fprintf(fileID,'\r\n');
fprintf(fileID,']\r\n\r\n');

fprintf(fileID,'RevenueOptional\t: [\r\n');
fprintf(fileID,'%d\t',Rev);
fprintf(fileID,'\r\n');
fprintf(fileID,']');

fclose(fileID);
