import java.io.IOException;

public class Main {
	public static void main(String[] args) throws IOException{
		String inputdata = "example.txt"; //Insert instance
		long runtime = 60000; //In milliseconds, 7200000 used in testing. 
		
		run2DRSSPHeuristic(inputdata,runtime); 
		//run2DRSSPHeuristic_GRASPonly(inputdata,runtime);
		
		//Results are save to: results_example.txt
		//Stowage plan is found in: stowageplan_example.txt
	}
	
	
	public static void run2DRSSPHeuristic(String inputdata, long runtime) throws IOException {
		// Initialize and set indices
		int nRows = ReadAndSet.nRowsRead(inputdata);
		int nColumns = ReadAndSet.nColumnsRead(inputdata);
		int nCargoesM = ReadAndSet.nCargoesMRead(inputdata);
		int nCargoesO = ReadAndSet.nCargoesORead(inputdata);
		int nPorts = ReadAndSet.nPortsRead(inputdata);
		int nCargoes = nCargoesM+nCargoesO;
		
		//Initialize Parameters
		int[] SquaresL = new int[nCargoes];
		int[] SquaresW = new int[nCargoes];
		int[] LoadingPort = new int[nCargoes];
		int[] UnloadingPort = new int[nCargoes];
		int[] NumberVehicle = new int[nCargoes];
		int[] CargoWeight = new int[nCargoes];
		int[] RevenueOptional = new int[nCargoesO];
		int[] DijkExit = new int[2]; //ExitSquare i,j for dijkstra calculations.
		int[][] UnusableSpace = new int[nRows][nColumns];
		int[][] ExitSquares = new int[nRows][nColumns];		
		int[][][] possibleSquares= new int[nRows][nColumns][nCargoes];
		int[][][] possibleSquaresWithExit = new int[nRows][nColumns][nCargoes];
		Cargo[] Cargoes = new Cargo[nCargoes];
		OptionalSet[] OptionalSets = new OptionalSet[(int) Math.pow(2,nCargoesO)];
		
		//Initialize Data
		ReadAndSet.setParameters(SquaresL,SquaresW,LoadingPort,UnloadingPort,NumberVehicle,CargoWeight,RevenueOptional,UnusableSpace,ExitSquares,DijkExit,inputdata);
		ReadAndSet.setCargoes(Cargoes, NumberVehicle, SquaresL, SquaresW, LoadingPort, UnloadingPort, CargoWeight, RevenueOptional, nCargoesM, nCargoesO);
		ReadAndSet.setPossibleSquares(possibleSquares, nCargoes, Cargoes, ExitSquares, UnusableSpace);
		ReadAndSet.setPossibleSquaresWithExit(possibleSquaresWithExit, nCargoes, Cargoes, ExitSquares, UnusableSpace);
		ReadAndSet.setOptionalSets(OptionalSets,nCargoesO,Cargoes,nCargoesM);
		
		//Set parameters
		int numberStowPlans = 1;
		//GRASP
		int[] InitialWeightParamSearchDir = {2000, 1000, 1000, 1000, 1000, 1000, 2000, 1000};
		int[] InitialWeightParamRandom = {1588, 1423, 1242, 1090, 958, 839, 735, 625, 565, 499, 432, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
		int[] InitialWeightParamCandidateListLength = {4632, 2372, 1465, 911, 621};
		int[] InitialWeightParamAreaOrSquares = {1000,1000};
		int deltaWeight = 50;
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//long maxruntimePrIter = 100000/nCargoesO;
		long maxruntimePrIter = 900000/nCargoesO;
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		
		
		//ALNS
		double[] InitialWeightDestroy = {1, 1, 1, 1, 1, 1};
		double[] InitialWeightRepair = {1, 1, 1, 1};
		double score1 = 33; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score2 = 13; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score3 = 9; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score4 = 0; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double temperature = 1;//BRUKES IKKE
		double scale = 0.005;//
		double coolingRate = 0.99975;//FIXED
		double r = 0.1;
		int sizeSegment = 150;//FIXED
		int weightResetSegments = 2000000;//FIXED
		long runTimeALNS = 30000;// FIXED: 2hours-time used by grasp SETTES NEDE I KODE
		double percentGrouping = 0.3;//FIXED
		double percentPlacement = 0.3;//FIXED
		long maxRunTimeRepair = 2000;//
		double maxDestroyPercentage = 1.0;//FIXED
		
		
		StowagePlan[] stowagePlans = new StowagePlan[numberStowPlans];
		long startTimeGRASP = System.currentTimeMillis();
		OptionalSelector1.run(nRows, nColumns, nCargoesM, nCargoesO, nPorts, UnusableSpace, possibleSquares, OptionalSets, Cargoes, stowagePlans, deltaWeight, maxruntimePrIter,InitialWeightParamSearchDir,InitialWeightParamRandom, InitialWeightParamCandidateListLength, InitialWeightParamAreaOrSquares);
		StowagePlan initialGRASP = stowagePlans[0];
		long TotaltimeGRASP = System.currentTimeMillis()-startTimeGRASP;
		System.out.println(TotaltimeGRASP);
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		runTimeALNS = runtime-TotaltimeGRASP;
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		
		
		
		ALNS improve = new ALNS(initialGRASP,DijkExit[0],DijkExit[1]);
		improve.runnew(Cargoes, OptionalSets, possibleSquares, possibleSquaresWithExit, UnusableSpace, ExitSquares, InitialWeightDestroy, InitialWeightRepair, score1, score2, score3, score4, temperature, coolingRate, r, sizeSegment, weightResetSegments, runTimeALNS, percentGrouping, percentPlacement, maxRunTimeRepair, deltaWeight, maxDestroyPercentage,InitialWeightParamSearchDir,InitialWeightParamRandom, InitialWeightParamCandidateListLength, InitialWeightParamAreaOrSquares, scale, inputdata);


		
	}
	

	public static void run2DRSSPHeuristic_GRASPonly(String inputdata, long runtime) throws IOException {
		// Initialize and set indices
		int nRows = ReadAndSet.nRowsRead(inputdata);
		int nColumns = ReadAndSet.nColumnsRead(inputdata);
		int nCargoesM = ReadAndSet.nCargoesMRead(inputdata);
		int nCargoesO = ReadAndSet.nCargoesORead(inputdata);
		int nPorts = ReadAndSet.nPortsRead(inputdata);
		int nCargoes = nCargoesM+nCargoesO;
		
		//Initialize Parameters
		int[] SquaresL = new int[nCargoes];
		int[] SquaresW = new int[nCargoes];
		int[] LoadingPort = new int[nCargoes];
		int[] UnloadingPort = new int[nCargoes];
		int[] NumberVehicle = new int[nCargoes];
		int[] CargoWeight = new int[nCargoes];
		int[] RevenueOptional = new int[nCargoesO];
		int[] DijkExit = new int[2]; //ExitSquare i,j for dijkstra calculations.
		int[][] UnusableSpace = new int[nRows][nColumns];
		int[][] ExitSquares = new int[nRows][nColumns];		
		int[][][] possibleSquares= new int[nRows][nColumns][nCargoes];
		int[][][] possibleSquaresWithExit = new int[nRows][nColumns][nCargoes];
		Cargo[] Cargoes = new Cargo[nCargoes];
		OptionalSet[] OptionalSets = new OptionalSet[(int) Math.pow(2,nCargoesO)];
		
		//Initialize Data
		ReadAndSet.setParameters(SquaresL,SquaresW,LoadingPort,UnloadingPort,NumberVehicle,CargoWeight,RevenueOptional,UnusableSpace,ExitSquares,DijkExit,inputdata);
		ReadAndSet.setCargoes(Cargoes, NumberVehicle, SquaresL, SquaresW, LoadingPort, UnloadingPort, CargoWeight, RevenueOptional, nCargoesM, nCargoesO);
		ReadAndSet.setPossibleSquares(possibleSquares, nCargoes, Cargoes, ExitSquares, UnusableSpace);
		ReadAndSet.setPossibleSquaresWithExit(possibleSquaresWithExit, nCargoes, Cargoes, ExitSquares, UnusableSpace);
		ReadAndSet.setOptionalSets(OptionalSets,nCargoesO,Cargoes,nCargoesM);
		
		//Set parameters
		int numberStowPlans = 1;
		//GRASP
		int[] InitialWeightParamSearchDir = {2000, 1000, 1000, 1000, 1000, 1000, 2000, 1000};
		int[] InitialWeightParamRandom = {1588, 1423, 1242, 1090, 958, 839, 735, 625, 565, 499, 432, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
		int[] InitialWeightParamCandidateListLength = {4632, 2372, 1465, 911, 621};
		int[] InitialWeightParamAreaOrSquares = {1000,1000};
		int deltaWeight = 50;
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//long maxruntimePrIter = 100000/nCargoesO;
		long maxruntimePrIter = 900000/nCargoesO;
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		
		
		//ALNS
		double[] InitialWeightDestroy = {1, 1, 1, 1, 1, 1};
		double[] InitialWeightRepair = {1, 1, 1, 1};
		double score1 = 33; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score2 = 13; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score3 = 9; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double score4 = 0; //ROPKE AND PISINGER (2006) & RIBEIRO AND LAPORTE (2012)
		double temperature = 1;//BRUKES IKKE
		double scale = 0;//
		double coolingRate = 0.99975;//FIXED
		double r = 0.1;
		int sizeSegment = 150;//FIXED
		int weightResetSegments = 2000000;//FIXED
		long runTimeALNS = 30000;// FIXED: 2hours-time used by grasp SETTES NEDE I KODE
		double percentGrouping = 0.3;//FIXED
		double percentPlacement = 0.3;//FIXED
		long maxRunTimeRepair = 2000;//
		double maxDestroyPercentage = 1.0;//FIXED
		
		
		StowagePlan[] stowagePlans = new StowagePlan[numberStowPlans];
		long startTimeGRASP = System.currentTimeMillis();
		OptionalSelector1.run(nRows, nColumns, nCargoesM, nCargoesO, nPorts, UnusableSpace, possibleSquares, OptionalSets, Cargoes, stowagePlans, deltaWeight, maxruntimePrIter,InitialWeightParamSearchDir,InitialWeightParamRandom, InitialWeightParamCandidateListLength, InitialWeightParamAreaOrSquares);
		StowagePlan initialGRASP = stowagePlans[0];
		long TotaltimeGRASP = System.currentTimeMillis()-startTimeGRASP;
		System.out.println(TotaltimeGRASP);
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		runTimeALNS = runtime-TotaltimeGRASP;
		
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$4
		
		
		
		ALNS improve = new ALNS(initialGRASP,DijkExit[0],DijkExit[1]);
		improve.runnewGRASPONLY(Cargoes, OptionalSets, possibleSquares, possibleSquaresWithExit, UnusableSpace, ExitSquares, InitialWeightDestroy, InitialWeightRepair, score1, score2, score3, score4, temperature, coolingRate, r, sizeSegment, weightResetSegments, runTimeALNS, percentGrouping, percentPlacement, maxRunTimeRepair, deltaWeight, maxDestroyPercentage,InitialWeightParamSearchDir,InitialWeightParamRandom, InitialWeightParamCandidateListLength, InitialWeightParamAreaOrSquares, scale, inputdata);


		
	}
	
}
