import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.nio.file.Files;
import java.util.Random;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.PriorityQueue;

class StowagePlan {
	int[][][] flowX;
	int[][][] flowXtemp;
	int[] cargoCount;
	int[][][][][] vehiclePlacement;
	int[] numPossibleSquaresCargo;
	int[][][] possibleSquares;
	int feasible;
	int nRows;
	int nColumns;
	int nCargoesM;
	int nCargoesO;
	int nCargoes;
	int nPorts;
	int maxVehicle;
	int areaNotPlaced;
	OptionalSet OptionalSetEvaluated;
	int optionalSetNumberEvaluated;
	VehiclePlacement[][] vehiclePlacements;
	int groupObjective;
	double placementObjective;
	int[][] squareCost;
	//Constructor
	StowagePlan(int row, int col, int nCargoesM, int nCargoesO, OptionalSet[] OptionalSets, int optionalSetEvaluated, int nPorts, Cargo[] Cargoes, int [][][]possibleSquares){
		this.flowX = new int[row][col][nCargoesM+nCargoesO];
		this.vehiclePlacements = new VehiclePlacement[row][col];
		this.cargoCount = new int [nCargoesM+nCargoesO];
		this.feasible = 0;
		this.nRows = row;
		this.nColumns = col;
		this.nCargoesM = nCargoesM;
		this.nCargoesO = nCargoesO;
		this.nCargoes = nCargoesM+nCargoesO;
		this.optionalSetNumberEvaluated = optionalSetEvaluated;
		this.OptionalSetEvaluated = OptionalSets[optionalSetEvaluated];
		this.nPorts = nPorts;
		int tempMaxVehicle = 0;
		for (int c = 0; c < nCargoes; c++) {
				if (tempMaxVehicle<Cargoes[c].NumberVehicle){
					tempMaxVehicle=Cargoes[c].NumberVehicle;
				}
		}
		//this.vehiclePlacement = new int[row][col][nCargoesM+nCargoesO][tempMaxVehicle][nPorts];
		this.maxVehicle = tempMaxVehicle;
		this.numPossibleSquaresCargo = new int[nCargoesM+nCargoesO];
		for (int c = 0; c < nCargoes; c++) {
			for (int i = 0; i < row; i++){
				for (int j = 0; j < col; j++){
					if (possibleSquares[i][j][c]==1){
						numPossibleSquaresCargo[c]++;
					}
				}
			}
		}
		this.possibleSquares = possibleSquares;
		
	}
	
	//Metoder for klassen
	
	
	

	void setSquareCost(int [][][]possibleSquares, Cargo[] Cargoes,int exitI, int exitJ, int[][] UnusableSpace, int[][] ExitSquares){
		//Setter squareCost
		squareCost = new int[nRows][nColumns];
		squareCost[exitI][exitJ] = 0;
		int maxSquareCost=0;
		
		int squareCostCounter = 0;
		for(int i = exitI; i < nRows; i++){
			squareCost[i][exitJ] = squareCostCounter;
			squareCostCounter++;
		}
		squareCostCounter = 0;
		for(int i = exitI; i >= 0; i--){
			squareCost[i][exitJ] = squareCostCounter;
			squareCostCounter++;
		}
		for(int i = 0; i < nRows; i++){
			squareCostCounter = squareCost[i][exitJ];
			for(int j = exitJ; j < nColumns; j++){
				squareCost[i][j] = squareCostCounter;
				squareCostCounter++;
				if (squareCost[i][j]>maxSquareCost  && UnusableSpace[i][j]==0 && ExitSquares[i][j]==0){
					maxSquareCost=squareCost[i][j];
				}
			}
			squareCostCounter = squareCost[i][exitJ];
			for(int j = exitJ; j >= 0; j--){
				squareCost[i][j] = squareCostCounter;
				squareCostCounter++;
				if (squareCost[i][j]>maxSquareCost  && UnusableSpace[i][j]==0 && ExitSquares[i][j]==0){
					maxSquareCost=squareCost[i][j];
				}
			}
		}
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				squareCost[i][j]=(squareCost[i][j]-maxSquareCost)*(-1);
			}
		}
		
		
		
		
		
		
	}

	void calculatePlacement(Cargo[] Cargoes){
		
		
		double counter=0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						double counter2=0;
						for(int i1 = i; i1 < i+Cargoes[c].SquaresL; i1++){
							for(int j1 = j; j1 < j+Cargoes[c].SquaresW; j1++){
								counter2 = counter2 + squareCost[i1][j1];
							}
						}
						counter2 = counter2/(Cargoes[c].SquaresL*Cargoes[c].SquaresL);
						counter2 = counter2*(Cargoes[c].UnloadingPort-Cargoes[c].LoadingPort);
						counter = counter + counter2;
					}
				}
			}
		}
		placementObjective = counter;
	}
	
	
	void calculateGrouing(Cargo[] Cargoes){
		int counter=0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						if (i<nRows-Cargoes[c].SquaresL && flowX[i+Cargoes[c].SquaresL][j][c]==1){
							counter++;
						}
						if (i>=Cargoes[c].SquaresL && flowX[i-Cargoes[c].SquaresL][j][c]==1){
							counter++;
						}
						if (j<nColumns-Cargoes[c].SquaresW && flowX[i][j+Cargoes[c].SquaresW][c]==1){
							counter++;
						}
						if (j>=Cargoes[c].SquaresW && flowX[i][j-Cargoes[c].SquaresW][c]==1){
							counter++;
						}
					}
				}
			}
		}
		groupObjective = counter;
	}
	
	
	void saveResultToXpress() throws IOException{
		File source = new File("data.txt");
		File dest = new File("dataoutput.txt");
		dest.delete();
		Files.copy(source.toPath(), dest.toPath());

		Writer output;
		output = new BufferedWriter(new FileWriter("dataoutput.txt", true));  
		output.append(System.getProperty( "line.separator" ));
		output.append(System.getProperty( "line.separator" ));
		output.append("FixedFlowX : [ ");
		for (int i = 0; i < nRows; i++) {
			for (int j = 0; j < nColumns; j++) {
				for (int c = 0; c < nCargoes; c++) {
					if (this.flowX[i][j][c]==1){
						String Str_i = Integer.toString(i+1);
						String Str_j = Integer.toString(j+1);
						String Str_c = Integer.toString(c+1);
						String print = "(" + Str_i + "," + Str_j + "," + Str_c + ") 1";
						output.append(System.getProperty( "line.separator" ));
						output.append(print);
					}
				}
			}
		}
		output.append(System.getProperty( "line.separator" ));
		output.append("]");
		output.append(System.getProperty( "line.separator" ));
		output.append(System.getProperty( "line.separator" ));
		output.append("FixedFlowY : [ ");
		output.append(System.getProperty( "line.separator" ));
		for (int y = 0; y < nCargoesO; y++) {
			if (this.OptionalSetEvaluated.CargoesCarried[y]==1){
				output.append("1 ");
			} else {
				output.append("0 ");
			}
		}
		output.append(System.getProperty( "line.separator" ));
		output.append("]");
		output.close();
	}
	
	void updateNumPossibleSquaresCargo(int i, int j, int[] numPossibleSquaresCargo1){
		for (int c = 0; c < nCargoes; c++) {
			if (possibleSquares[i][j][c]==1){
				numPossibleSquaresCargo1[c]--;
			}
		}
	}
	
	void updateIJsearchDir(int[] arrI, int [] arrJ, int[] searchDir, int squareCounter){
		//Bottom left corner, row, col
		int i = arrI[0];
		int j = arrJ[0];
		
		if (searchDir[0]==1){
			if (squareCounter == nRows*nColumns){
				i = 0;
				j = -1;
			}
			j++;
			if (j==nColumns){
				j=0;
				i++;
			}
		}
		//Bottom left corner, col, row
		if (searchDir[1]==1){
			if (squareCounter == nRows*nColumns){
				i = -1;
				j = 0;
			}
			i++;
			if (i==nRows){
				i=0;
				j++;
			}
		}
		//Upper left corner, row,col
		if (searchDir[2]==1){
			if (squareCounter == nRows*nColumns){
				i = nRows-1;
				j = -1;
			}
			j++;
			if (j==nColumns){
				j=0;
				i--;
			}
		}
		//Upper left corner, col,row
		if (searchDir[3]==1){
			if (squareCounter == nRows*nColumns){
				i = nRows;
				j = 0;
			}
			i--;
			if (i==-1){
				i=nRows-1;
				j++;
			}
		}
		//Upper right corner, row,col
		if (searchDir[4]==1){
			if (squareCounter == nRows*nColumns){
				i = nRows-1;
				j = nColumns;
			}
			j--;
			if (j==-1){
				j=nColumns-1;
				i--;
			}
		}
		//Upper right corner, col,row
		if (searchDir[5]==1){
			if (squareCounter == nRows*nColumns){
				i = nRows;
				j = nColumns-1;
			}
			i--;
			if (i==-1){
				i=nRows-1;
				j--;
			}
		}
		//Bottom right corner, row,col
		if (searchDir[6]==1){
			if (squareCounter == nRows*nColumns){
				i = 0;
				j = nColumns;
			}
			j--;
			if (j==-1){
				j=nColumns-1;
				i++;
			}
		}
		//Bottom right corner, col,row
		if (searchDir[7]==1){
			if (squareCounter == nRows*nColumns){
				i = -1;
				j = nColumns-1;
			}
			i++;
			if (i==nRows){
				i=0;
				j--;
			}
		}
		
		arrI[0] = i;
		arrJ[0] = j;
	}
	
	void GRASPconstruction(int [][][]possibleSquares, Cargo[] Cargoes, int searchDirParam, double randomParam, int candidateListLengthInput, int areaOrSquares){
		int[][] takenSquares = new int[nRows][nColumns];
		int[] numberOfVehiclePlaced = new int[nCargoes];
		int tempCargo = -1;
		int sum = 0;
		int[][][]flowX = new int [nRows][nColumns][nCargoes];
		
		
		//RANDOM
		Random r = new Random();
		double randomChooser;
		double possibleCargoCounter = 0;
		double chooserRandomCounter = 1;
		int[] possibleCargo = new int [nCargoes];
		
		
		//SearchDir
		int[] searchDir = new int [8];
		searchDir[searchDirParam]=1;
		int squareCounter = nRows*nColumns;
		int i = 0;
		int j = 0;
		int[] arrI = new int [1];
		int[] arrJ = new int [1];
		arrI[0] = 0;
		arrJ[0] = 0;
		
		//Candidatelist
		
		int[] numPossibleSquaresCargo = new int [nCargoes];
		for (int c = 0; c < nCargoes; c++) {
			numPossibleSquaresCargo[c] = this.numPossibleSquaresCargo[c];
		}
		
		
		int candidateListMaxLength = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				candidateListMaxLength++;
			}
		}
		int candidateListLength = Math.min(candidateListLengthInput, candidateListMaxLength);
		
		int[] placedCandidate = new int [nCargoes];
		
		int[][] candidateListArea = new int [nCargoes][2];
		
		int[][] candidateListSquares = new int [nCargoes][2];
		int numberCargoesInCandidateList;
		while (squareCounter>0){
			updateIJsearchDir(arrI,arrJ,searchDir,squareCounter);
			i = arrI[0];
			j = arrJ[0];
			// Set parameters to startvalue
			for (int c = 0; c < nCargoes; c++) {
				possibleCargo[c]=0; 
				candidateListArea[c][0]=0;
				candidateListArea[c][1]=-1;
				candidateListSquares[c][0]=nRows*nColumns;
				candidateListSquares[c][1]=-1;
				placedCandidate[c]=0;
			}
			possibleCargoCounter=0;
			numberCargoesInCandidateList = 0;
			
			
			
			if (takenSquares[i][j]==0){ //Str det noe p denne plassen
				
				for (int c = 0; c < nCargoes; c++) {
					// Check possible cargoes
					if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){ //Skal cargoen plasseres?
						if (numberOfVehiclePlaced[c]<Cargoes[c].NumberVehicle){ //Er alle bilene plassert?
							//Bottom left corner
							if (searchDir[0]==1 || searchDir[1]==1){
								if (possibleSquares[i][j][c]==1){
									for (int a = i; a < i +Cargoes[c].SquaresL; a++) {
										for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Upper left corner
							if (searchDir[2]==1 || searchDir[3]==1){
								if ((i-Cargoes[c].SquaresL+1)>=0 && possibleSquares[i-Cargoes[c].SquaresL+1][j][c]==1){
									for (int a = i; a > i - Cargoes[c].SquaresL; a--) {
										for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Upper right corner
							if (searchDir[4]==1 || searchDir[5]==1){
								if ((i-Cargoes[c].SquaresL+1)>=0 && (j-Cargoes[c].SquaresW+1>=0) && possibleSquares[i-Cargoes[c].SquaresL+1][j-Cargoes[c].SquaresW+1][c]==1){
									for (int a = i; a > i - Cargoes[c].SquaresL; a--) {
										for (int b = j; b > j - Cargoes[c].SquaresW; b--) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Bottom right corner
							if (searchDir[6]==1 || searchDir[7]==1){
								if ((j-Cargoes[c].SquaresW+1)>=0 && possibleSquares[i][j-Cargoes[c].SquaresW+1][c]==1){
									for (int a = i; a < i + Cargoes[c].SquaresL; a++) {
										for (int b = j; b > j - Cargoes[c].SquaresW; b--) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
								
							
						}
					}
					sum=0;
				} // Her slutter Cargoloopen
				// Update Canidatelist:
				
				int[] candidateListCounter = new int [candidateListLength];
				for (int c = 0; c < nCargoes; c++) {
					if (possibleCargo[c]==1 && takenSquares[i][j]==0){
						if (candidateListArea[0][0]<Cargoes[c].Area){
							candidateListArea[0][0]=Cargoes[c].Area;
							candidateListArea[0][1]=c;
							candidateListCounter[0]=1;
						}
						if (candidateListSquares[0][0]>numPossibleSquaresCargo[c]){
							candidateListSquares[0][0]=numPossibleSquaresCargo[c];
							candidateListSquares[0][1]=c;
							candidateListCounter[0]=1;
						}
					}
				}
				if (areaOrSquares==0 && candidateListCounter[0]==1){
					placedCandidate[candidateListArea[0][1]]=1;
					numberCargoesInCandidateList++;
				} 
				if (areaOrSquares==1 && candidateListCounter[0]==1){
					placedCandidate[candidateListSquares[0][1]]=1;
					numberCargoesInCandidateList++;
				}
				
				for (int n = 1; n < candidateListLength; n++) {
					for (int c = 0; c < nCargoes; c++) {
						if (possibleCargo[c]==1 && takenSquares[i][j]==0){
							if (candidateListArea[n][0]<Cargoes[c].Area && placedCandidate[c]==0 && areaOrSquares==0){
								candidateListArea[n][0]=Cargoes[c].Area;
								candidateListArea[n][1]=c;
								candidateListCounter[n]=1;
							}
							if (candidateListSquares[n][0]>numPossibleSquaresCargo[c] && placedCandidate[c]==0 && areaOrSquares==1){
								candidateListSquares[n][0]=numPossibleSquaresCargo[c];
								candidateListSquares[n][1]=c;
								candidateListCounter[n]=1;
							}
						}
					}
					if (areaOrSquares==0 && candidateListCounter[n]==1){
						placedCandidate[candidateListArea[n][1]]=1;
						numberCargoesInCandidateList++;
					} 
					if (areaOrSquares==1 && candidateListCounter[n]==1){
						placedCandidate[candidateListSquares[n][1]]=1;
						numberCargoesInCandidateList++;
					}
				}
							
				//Choose which cargo to place
				double cargoChooser;
				int chooserCargoCounter = 1;
				double numberCargoesInCandidateListDouble = numberCargoesInCandidateList;
				cargoChooser = r.nextDouble();
				for (int n = 0; n < numberCargoesInCandidateList; n++) {
					if (cargoChooser<chooserCargoCounter/numberCargoesInCandidateListDouble){
						if (areaOrSquares==0) {
							tempCargo = candidateListArea[n][1];
						} else {
							tempCargo = candidateListSquares[n][1];
						}
						break;
					} else {
						chooserCargoCounter++;
					}
				}
				
				
				//Random override if randomParam is fullfilled
				chooserRandomCounter = 1;
				randomChooser = r.nextDouble();
				if (r.nextDouble()<randomParam){
					for (int c = 0; c < nCargoes; c++) {
						if (possibleCargo[c]==1){
							if (randomChooser<chooserRandomCounter/possibleCargoCounter ){
								tempCargo = c;
								break;
							} else {
								chooserRandomCounter++;
							}
						}
					}
				}
				///
				if (tempCargo!=-1){
					numberOfVehiclePlaced[tempCargo]++;
					if (searchDir[0]==1 || searchDir[1]==1){
						flowX[i][j][tempCargo]=1;
						for (int a = i; a < i +Cargoes[tempCargo].SquaresL; a++) {
							for (int b = j; b < j + Cargoes[tempCargo].SquaresW; b++) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[2]==1 || searchDir[3]==1){
						flowX[i-Cargoes[tempCargo].SquaresL+1][j][tempCargo]=1;
						for (int a = i; a > i - Cargoes[tempCargo].SquaresL; a--) {
							for (int b = j; b < j + Cargoes[tempCargo].SquaresW; b++) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[4]==1 || searchDir[5]==1){
						flowX[i-Cargoes[tempCargo].SquaresL+1][j-Cargoes[tempCargo].SquaresW+1][tempCargo]=1;
						for (int a = i; a > i - Cargoes[tempCargo].SquaresL; a--) {
							for (int b = j; b > j - Cargoes[tempCargo].SquaresW; b--) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[6]==1 || searchDir[7]==1){
						flowX[i][j-Cargoes[tempCargo].SquaresW+1][tempCargo]=1;
						for (int a = i; a < i + Cargoes[tempCargo].SquaresL; a++) {
							for (int b = j; b > j - Cargoes[tempCargo].SquaresW; b--) {
								takenSquares[a][b]=1;
							}
						}
					}
				}
				
			}
			updateNumPossibleSquaresCargo(i,j,numPossibleSquaresCargo);
			tempCargo = -1;									
																
																
			//System.out.println("i = " + i + ", j = " + j + "for cargo 0: " + candidateListSquares[0][1] + "," + candidateListSquares[1][1] + "," + candidateListSquares[2][1] + "," + candidateListSquares[3][1] + "numberinlist: "+ numberCargoesInCandidateList);												
			squareCounter--;			
		}
		
		//Sjekker om lsning er gyldig.
		
		int counterCargoes = 0;
		int counterPlaced = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				if (numberOfVehiclePlaced[c]==Cargoes[c].NumberVehicle){
					counterPlaced++;
				}
				counterCargoes++;
			}
		}
		int tempAreaOverusage = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				tempAreaOverusage = tempAreaOverusage + (Cargoes[c].NumberVehicle-numberOfVehiclePlaced[c])*Cargoes[c].Area;
			}
		}
		
		if (counterCargoes == counterPlaced){
			feasible = 1;
			this.flowX = flowX;
			areaNotPlaced = 0;
		} else {
			feasible = 0;
			areaNotPlaced = tempAreaOverusage;
			//this.flowX = flowX;
		}
	}
	
	void printVehiclePlacementToFile(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares, String input) throws FileNotFoundException, UnsupportedEncodingException{
		String stowageplan = "stowageplan_" + input;
		PrintWriter writer = new PrintWriter(stowageplan, "UTF-8");
		int counter = 0;
		for (int i = 0; i < nColumns; i++) {
			writer.print("----");
		}
		writer.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int a = 0; a < 2; a++) {
				for (int j = 0; j < nColumns; j++) {
					writer.print("|");
					if (vehiclePlacements[i][j].empty==0 && a==0){
						String c1 = String.format("%02d", vehiclePlacements[i][j].cargo);
						writer.print("c" + (c1));
						counter++;
					}
					if (vehiclePlacements[i][j].empty==0 && a==1){
						String v1 = String.format("%02d", vehiclePlacements[i][j].vehicle);
						writer.print("v" + v1);
						counter++;
					}
					if (UnusableSpace[i][j] == 1){
						writer.print("XX ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						writer.print("EX ");
						counter++;
					}
					if (counter==0){
						writer.print("   ");
					}
					counter = 0;
				}
				writer.print("|");
				if (a==0){
				writer.println();
				}
			}
			writer.println();
			for (int i1 = 0; i1 < nColumns; i1++) {
				writer.print("----");
			}
			writer.println();
		}
		writer.close();
	}


	void printVehiclePlacementToFileGRASPONLY(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares, String input) throws FileNotFoundException, UnsupportedEncodingException{
		String stowageplan = "stowageplan_GRASPONLY_" + input;
		PrintWriter writer = new PrintWriter(stowageplan, "UTF-8");
		int counter = 0;
		for (int i = 0; i < nColumns; i++) {
			writer.print("----");
		}
		writer.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int a = 0; a < 2; a++) {
				for (int j = 0; j < nColumns; j++) {
					writer.print("|");
					if (vehiclePlacements[i][j].empty==0 && a==0){
						String c1 = String.format("%02d", vehiclePlacements[i][j].cargo);
						writer.print("c" + (c1));
						counter++;
					}
					if (vehiclePlacements[i][j].empty==0 && a==1){
						String v1 = String.format("%02d", vehiclePlacements[i][j].vehicle);
						writer.print("v" + v1);
						counter++;
					}
					if (UnusableSpace[i][j] == 1){
						writer.print("XX ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						writer.print("EX ");
						counter++;
					}
					if (counter==0){
						writer.print("   ");
					}
					counter = 0;
				}
				writer.print("|");
				if (a==0){
				writer.println();
				}
			}
			writer.println();
			for (int i1 = 0; i1 < nColumns; i1++) {
				writer.print("----");
			}
			writer.println();
		}
		writer.close();
	}

	
	void printVehiclePlacementToFileAltpaenlinje(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares) throws FileNotFoundException, UnsupportedEncodingException{
		PrintWriter writer = new PrintWriter("output.txt", "UTF-8");
		int counter = 0;
		for (int i = 0; i < nColumns; i++) {
			writer.print("-------");
		}
		writer.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int j = 0; j < nColumns; j++) {
				writer.print("|");
				if (vehiclePlacements[i][j].empty==0){
					String c1 = String.format("%02d", vehiclePlacements[i][j].cargo);
					String v1 = String.format("%02d", vehiclePlacements[i][j].vehicle);
					writer.print("c" + (c1) + "v" + v1);
					counter++;
				}
				if (UnusableSpace[i][j] == 1){
					writer.print("  XX  ");
					counter++;
				}
				if (ExitSquares[i][j] == 1){
					writer.print(" EXIT ");
					counter++;
				}
				if (counter==0){
					writer.print("      ");
				}
				counter = 0;
			}
			writer.print("|");
			writer.println();
			for (int i1 = 0; i1 < nColumns; i1++) {
				writer.print("-------");
			}
			writer.println();
		}
		writer.close();
	}

	void printFlowXToFile(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares) throws FileNotFoundException, UnsupportedEncodingException{
		PrintWriter writer = new PrintWriter("output.txt", "UTF-8");
		int counter = 0;
		for (int i = 0; i < nColumns; i++) {
			writer.print("------");
		}
		writer.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int j = 0; j < nColumns; j++) {
				writer.print("|");
				for (int c = 0; c < nCargoes; c++) {
					if (flowX[i][j][c]==1){
						String c1 = String.format("%02d", c);
						writer.print(" c" + (c1) + " ");
						counter++;
					}
					
				}
				if (UnusableSpace[i][j] == 1){
					writer.print(" XX  ");
					counter++;
				}
				if (ExitSquares[i][j] == 1){
					writer.print(" EXIT");
					counter++;
				}
				if (counter==0){
					writer.print("     ");
				}
				counter = 0;
			}
			writer.print("|");
			writer.println();
			for (int i1 = 0; i1 < nColumns; i1++) {
				writer.print("------");
			}
			writer.println();
		}
		writer.close();
	}
	
	void printFlowX(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares){
		int counter = 0;
		for (int i = 0; i < nColumns; i++) {
			System.out.print("------");
		}
		System.out.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int j = 0; j < nColumns; j++) {
				System.out.print("|");
				for (int c = 0; c < nCargoes; c++) {
					if (flowX[i][j][c]==1){
						String c1 = String.format("%02d", c);
						System.out.print(" c" + (c1) + " ");
						counter++;
					}
					
				}
				if (UnusableSpace[i][j] == 1){
					System.out.print(" XX  ");
					counter++;
				}
				if (ExitSquares[i][j] == 1){
					System.out.print(" EXIT");
					counter++;
				}
				if (counter==0){
					System.out.print("     ");
				}
				counter = 0;
			}
			System.out.print("|");
			System.out.println();
			for (int i1 = 0; i1 < nColumns; i1++) {
				System.out.print("------");
			}
			System.out.println();
		}
	}

	void setVehiclePlacement(Cargo[] Cargoes) {
		int[] vehicleNumber;
		vehicleNumber = new int[flowX[0][0].length];
		this.vehiclePlacement = new int[nRows][nColumns][nCargoesM+nCargoesO][maxVehicle][nPorts];
		
		
		
		
		for (int i = 0; i < flowX.length; i++) {
			for (int j = 0; j < flowX[0].length; j++) {
				for (int c = 0; c < flowX[0][0].length; c++) {
					if (flowX[i][j][c]==1){
						for (int a = i; a < i+Cargoes[c].SquaresL; a++) {
							for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
								for (int p = Cargoes[c].LoadingPort; p <= Cargoes[c].UnloadingPort-1; p++) {
									vehiclePlacement[a][b][c][vehicleNumber[c]][p]=1;
								}
							}
						}
						vehicleNumber[c]++;
					}
				}
			}
		}
	}

	void setNewVehiclePlacement(Cargo[] Cargoes) {
		int[] vehicleNumber;
		vehicleNumber = new int[flowX[0][0].length];
		
		
		for (int i = 0; i < flowX.length; i++) {
			for (int j = 0; j < flowX[0].length; j++) {
				vehiclePlacements[i][j] = new VehiclePlacement(i,j,nPorts);
			}
		}
		
		for (int i = 0; i < flowX.length; i++) {
			for (int j = 0; j < flowX[0].length; j++) {
				for (int c = 0; c < flowX[0][0].length; c++) {
					if (flowX[i][j][c]==1){
						for (int a = i; a < i+Cargoes[c].SquaresL; a++) {
							for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
								vehiclePlacements[a][b].cargo=c;
								vehiclePlacements[a][b].vehicle=vehicleNumber[c];
								vehiclePlacements[a][b].setPorts(Cargoes);
							}
						}
						vehicleNumber[c]++;
					}
				}
			}
		}
		
	}
	
	void printNewVehiclePlacement(int[][] ExitSquares, int[][]UnusableSpace, Cargo[] Cargoes) {
		int counter = 0;
		for (int p = 1; p < nPorts; p++) {	
			System.out.println("For port number: " + (p+1));
			for (int i1 = 0; i1 < nColumns; i1++) {
				System.out.print("---------");
			}
			System.out.println();
			for (int i = nRows-1; i >= 0; i--) {//
				for (int j = 0; j < nColumns; j++) {
					System.out.print("|");
					if (vehiclePlacements[i][j].empty==0 && vehiclePlacements[i][j].isOnShipInPort[p]==1){
						String c1 = String.format("%02d", vehiclePlacements[i][j].cargo);
						String v1 = String.format("%02d", vehiclePlacements[i][j].vehicle);
						System.out.print(" c" + (c1) + "v" + v1 +" ");
						counter++;
					}
					if (UnusableSpace[i][j] == 1){
						System.out.print("   XX   ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						System.out.print("  EXIT  ");
						counter++;
					}
					if (counter==0){
						System.out.print("        ");
					}
					counter = 0;
				}
				System.out.print("|");
				System.out.println();
				for (int i1 = 0; i1 < nColumns; i1++) {
					System.out.print("---------");
				}
				System.out.println();
			}
			System.out.println();
		}
		
	}
	
	void printVehiclePlacement(int[][] ExitSquares, int[][]UnusableSpace, Cargo[] Cargoes) {
		int counter = 0;
		for (int p = 1; p < nPorts; p++) {	
			System.out.println("For port number: " + (p+1));
			for (int i1 = 0; i1 < nColumns; i1++) {
				System.out.print("---------");
			}
			System.out.println();
			for (int i = nRows-1; i >= 0; i--) {//
				for (int j = 0; j < nColumns; j++) {
					System.out.print("|");
					for (int c = 0; c < nCargoes; c++) {
						for (int v = 0; v < maxVehicle; v++) {
							if (vehiclePlacement[i][j][c][v][p]==1){
								String c1 = String.format("%02d", c);
								String v1 = String.format("%02d", v);
								System.out.print(" c" + (c1) + "v" + v1 +" ");
								counter++;
							}
						}
					}
					if (UnusableSpace[i][j] == 1){
						System.out.print("   XX   ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						System.out.print("  EXIT  ");
						counter++;
					}
					if (counter==0){
						System.out.print("        ");
					}
					counter = 0;
				}
				System.out.print("|");
				System.out.println();
				for (int i1 = 0; i1 < nColumns; i1++) {
					System.out.print("---------");
				}
				System.out.println();
			}
			System.out.println();
		}
		
	}

	void printNewVehiclePlacementLastLoading(int[][] ExitSquares, int[][]UnusableSpace, Cargo[] Cargoes) {
		int counter = 0;
		int lastLoadingPort = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (Cargoes[c].LoadingPort > lastLoadingPort){
				lastLoadingPort = Cargoes[c].LoadingPort;
			}
		}
		for (int p = lastLoadingPort; p <= lastLoadingPort; p++) {	
			System.out.println("Leg between last loading port and first unloading port");
			for (int i1 = 0; i1 < nColumns; i1++) {
				System.out.print("---------");
			}
			System.out.println();
			for (int i = nRows-1; i >= 0; i--) {//
				for (int j = 0; j < nColumns; j++) {
					System.out.print("|");
					if (vehiclePlacements[i][j].empty==0 && vehiclePlacements[i][j].isOnShipInPort[p]==1){
						String c1 = String.format("%02d", vehiclePlacements[i][j].cargo);
						String v1 = String.format("%02d", vehiclePlacements[i][j].vehicle);
						System.out.print(" c" + (c1) + "v" + v1 +" ");
						counter++;
					}
					if (UnusableSpace[i][j] == 1){
						System.out.print("   XX   ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						System.out.print("  EXIT  ");
						counter++;
					}
					if (counter==0){
						System.out.print("        ");
					}
					counter = 0;
				}
				System.out.print("|");
				System.out.println();
				for (int i1 = 0; i1 < nColumns; i1++) {
					System.out.print("---------");
				}
				System.out.println();
			}
			System.out.println();
		}
		
	}
	
	void printVehiclePlacementLastLoading(int[][] ExitSquares, int[][]UnusableSpace, Cargo[] Cargoes) {
		int counter = 0;
		int lastLoadingPort = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (Cargoes[c].LoadingPort > lastLoadingPort){
				lastLoadingPort = Cargoes[c].LoadingPort;
			}
		}
		for (int p = lastLoadingPort; p <= lastLoadingPort; p++) {	
			System.out.println("Leg between last loading port and first unloading port");
			for (int i1 = 0; i1 < nColumns; i1++) {
				System.out.print("---------");
			}
			System.out.println();
			for (int i = nRows-1; i >= 0; i--) {//
				for (int j = 0; j < nColumns; j++) {
					System.out.print("|");
					for (int c = 0; c < nCargoes; c++) {
						for (int v = 0; v < maxVehicle; v++) {
							if (vehiclePlacement[i][j][c][v][p]==1){
								String c1 = String.format("%02d", c);
								String v1 = String.format("%02d", v);
								System.out.print(" c" + (c1) + "v" + v1 +" ");
								counter++;
							}
						}
					}
					if (UnusableSpace[i][j] == 1){
						System.out.print("   XX   ");
						counter++;
					}
					if (ExitSquares[i][j] == 1){
						System.out.print("  EXIT  ");
						counter++;
					}
					if (counter==0){
						System.out.print("        ");
					}
					counter = 0;
				}
				System.out.print("|");
				System.out.println();
				for (int i1 = 0; i1 < nColumns; i1++) {
					System.out.print("---------");
				}
				System.out.println();
			}
			System.out.println();
		}
		
	}
	
	void printFlowXtemp(Cargo[] Cargoes, int[][] UnusableSpace, int[][] ExitSquares){
		int counter = 0;
		for (int i = 0; i < flowX.length; i++) {
			System.out.print("------");
		}
		System.out.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int j = 0; j < nColumns; j++) {
				System.out.print("|");
				for (int c = 0; c < nCargoes; c++) {
					if (flowXtemp[i][j][c]==1){
						String c1 = String.format("%02d", c);
						System.out.print(" c" + (c1) + " ");
						counter++;
					}
					
				}
				if (UnusableSpace[i][j] == 1){
					System.out.print(" XX  ");
					counter++;
				}
				if (ExitSquares[i][j] == 1){
					System.out.print(" EXIT");
					counter++;
				}
				if (counter==0){
					System.out.print("     ");
				}
				counter = 0;
			}
			System.out.print("|");
			System.out.println();
			for (int i1 = 0; i1 < nRows; i1++) {
				System.out.print("------");
			}
			System.out.println();
		}
	}

	void destroyOperator1(double r, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int removeNumVehicle = 0;
		int totalNumVehicle =  0;
		
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				totalNumVehicle = totalNumVehicle + Cargoes[c].NumberVehicle;
			}
		}
		
		int[] selectI = new int[totalNumVehicle];
		int[] selectJ = new int[totalNumVehicle];
		int[] selectC = new int[totalNumVehicle];
		//int[] shuffleN = new int[n];
		int n = 0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						selectI[n] = i;
						selectJ[n] = j;
						selectC[n] = c;
						n++;
					}
				}
			}
		}
		if (n!=0){
			List<Integer> shuffleN = new ArrayList<>();
			for(int count = 0; count < n; count++){
				shuffleN.add(count);
			}
			Collections.shuffle(shuffleN);

			
			removeNumVehicle = (int) Math.ceil(totalNumVehicle*r);

			int temp = 0;
			for(int a = 0; a < removeNumVehicle; a++){
				temp = shuffleN.get(a);
				flowXtemp[selectI[temp]][selectJ[temp]][selectC[temp]] = 0;
			}
		}
	}

	void destroyOperator2(double[] cargoRandom, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int removeNumVehicle = 0;
		for(int c = 0; c < cargoRandom.length; c++){
			if((cargoRandom[c] > 0) && (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
				removeNumVehicle = (int) Math.ceil(Cargoes[c].NumberVehicle*cargoRandom[c]);
				
				int[] selectI = new int[Cargoes[c].NumberVehicle];
				int[] selectJ = new int[Cargoes[c].NumberVehicle];
				
				int n = 0;
				for(int i = 0; i < nRows; i++){
					for(int j = 0; j < nColumns; j++){
						if(flowX[i][j][c] == 1){
							selectI[n] = i;
							selectJ[n] = j;
							n++;
						}
					}
				}
				if (n!=0){
					List<Integer> shuffleN = new ArrayList<>();
					for(int count = 0; count < n; count++){
						shuffleN.add(count);
					}
					Collections.shuffle(shuffleN);

					for(int a = 0; a < removeNumVehicle; a++){
						flowXtemp[selectI[shuffleN.get(a)]][selectJ[shuffleN.get(a)]][c] = 0;
					}	
				}
				
			}		
		}
		
		
		
	}
	
	void destroyOperator3(int port, double[] cargoRandom, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int removeNumVehicle = 0;
		for(int c = 0; c < cargoRandom.length; c++){
			if(cargoRandom[c] > 0 && (Cargoes[c].LoadingPort == port || Cargoes[c].UnloadingPort == port) && (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
				removeNumVehicle = (int) Math.ceil(Cargoes[c].NumberVehicle*cargoRandom[c]);
				
				int[] selectI = new int[Cargoes[c].NumberVehicle];
				int[] selectJ = new int[Cargoes[c].NumberVehicle];
				
				int n = 0;
				for(int i = 0; i < nRows; i++){
					for(int j = 0; j < nColumns; j++){
						if(flowX[i][j][c] == 1){
							selectI[n] = i;
							selectJ[n] = j;
							n++;
						}
					}
				}
				
				if (n!=0){
					List<Integer> shuffleN = new ArrayList<>();
					for(int count = 0; count < n; count++){
						shuffleN.add(count);
					}
					
						Collections.shuffle(shuffleN);
					
					

					for(int a = 0; a < removeNumVehicle; a++){
						flowXtemp[selectI[shuffleN.get(a)]][selectJ[shuffleN.get(a)]][c] = 0;
					}
				}
			}		
		}
	}
	
	void destroyOperator4(int i_start, int j_start, int i_end, int j_end, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1 && ((i < i_start || i > i_end) || (j < j_start || j > j_end))){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
	}

	void destroyOperator5(Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int n = 0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(Cargoes[c].NumberVehicle > 1){
						if(i-Cargoes[c].SquaresL >= 0 && flowX[i][j][c] == 1 && flowX[i-Cargoes[c].SquaresL][j][c] == 1){
							n++;
						}
						if(i+Cargoes[c].SquaresL < nRows && flowX[i][j][c] == 1 && flowX[i+Cargoes[c].SquaresL][j][c] == 1){
							n++;
						}
						if(j-Cargoes[c].SquaresW >= 0 && flowX[i][j][c] == 1 && flowX[i][j-Cargoes[c].SquaresW][c] == 1){
							n++;
						}
						if(j+Cargoes[c].SquaresW < nColumns && flowX[i][j][c] == 1 && flowX[i][j+Cargoes[c].SquaresW][c] == 1){
							n++;
						}
						if(n == 0){
							flowXtemp[i][j][c] = 0;
						}
						
						n = 0;
					}
				}
			}
		}
		
		
	}
	
	void destroyOperator6(double prcShiftCost, double prcRouteCost, double prcNumShift, ShiftingResult shiftRes, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		setNewVehiclePlacement(Cargoes);
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int totalVehOnDeck = 0;
		
		for(int c = 0; c < nCargoes; c++){
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))
				totalVehOnDeck = totalVehOnDeck + Cargoes[c].NumberVehicle;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//     Remove a given percentage of the vehicles that have the highest shifting cost for a single vehicle
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		int numVehRemShift = (int) Math.ceil(totalVehOnDeck*prcShiftCost);
		
		List<Double> sortedListVehicleShiftCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleShiftCost.add(shiftRes.shiftCostCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.shiftCostCV[c][v] + "  ");
				}
			}
		}

		//System.out.println();
		//System.out.println(sortedListVehicleShiftCost.size() + " HEIHEIEHEIE");
		Collections.sort(sortedListVehicleShiftCost);
		Collections.reverse(sortedListVehicleShiftCost);
		
		double[] temp_shiftCost = new double[Math.min(numVehRemShift, sortedListVehicleShiftCost.size())];
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			temp_shiftCost[i] = sortedListVehicleShiftCost.get(i);
		}
		
		/*
		
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleShiftCost.size(); i++){
			System.out.print(sortedListVehicleShiftCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int a = 0; a < temp_shiftCost.length; a++){
							if(shiftRes.shiftCostCV[c1][v1] == temp_shiftCost[a] && flowXtemp[i][j][c1]==1 &&  numVehRemShift > 0){
								flowXtemp[i][j][c1] = 0;
								numVehRemShift = numVehRemShift - 1;
								temp_shiftCost[a]=-1;
								//System.out.print(temp_shiftCost[a] + " ");
							}
						}
					}
				}
			}
		}
									
									
		

		//System.out.println();
		//System.out.println();
			
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//	
		//                  Remove vehicles with the highest shifting cost for a given route. 
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
			
		int numVehRemRoute = (int) Math.ceil(totalVehOnDeck*prcRouteCost);
		
		List<Double> sortedListVehicleRouteCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				for(int p = 0; p < 2; p++){
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.routeCost[c][v][p] > 0.5){
						sortedListVehicleRouteCost.add(shiftRes.routeCost[c][v][p]);	
						//System.out.println(shiftRes.routeCost[c][v][p] + " ");
					}
				}
			}
		}
		//System.out.println();
		//System.out.println();
		//System.out.println(sortedListVehicleRouteCost.size() + " OKOK");
		Collections.sort(sortedListVehicleRouteCost);
		Collections.reverse(sortedListVehicleRouteCost);
		
		double[] temp_RouteCost = new double[Math.min(numVehRemRoute, sortedListVehicleRouteCost.size())];
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			temp_RouteCost[i] = sortedListVehicleRouteCost.get(i);
		}
	
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		
		System.out.println();
		for(int i = 0; i < sortedListVehicleRouteCost.size(); i++){
			System.out.print(sortedListVehicleRouteCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int p = 0; p < 2; p++){
							for(int a = 0; a < temp_RouteCost.length; a++){
								if(shiftRes.routeCost[c1][v1][p] == temp_RouteCost[a] && flowXtemp[i][j][c1]==1 && numVehRemRoute > 0){
									//System.out.print(temp_RouteCost[a] + " ");
									flowXtemp[i][j][c1] = 0;
									numVehRemRoute = numVehRemRoute - 1;
									temp_RouteCost[a]=-1;
								}
							}
						}
					}
				}
			}
		}
											
			
		
		
		//System.out.println();
		//System.out.println();
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//																												//
		//                  Remove vehicles with the highest number of other vehicles that move through it				// 
		//																												//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
		
		int numVehRemNumShift = (int) Math.ceil(totalVehOnDeck*prcNumShift);
		
		List<Double> sortedListVehicleNumShift = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleNumShift.add(shiftRes.numShiftCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.numShiftCV[c][v] + "   ");
				}
			}
		}

		Collections.sort(sortedListVehicleNumShift);
		Collections.reverse(sortedListVehicleNumShift);
		
		double[] temp_numShift = new double[Math.min(numVehRemNumShift, sortedListVehicleNumShift.size())];
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			temp_numShift[i] = sortedListVehicleNumShift.get(i);
		}
		
		
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleNumShift.size(); i++){
			System.out.print(sortedListVehicleNumShift.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int q = 0; q < nPorts; q++){
							if (vehiclePlacements[i][j].isOnShipInPort[q] == 1){
								for(int a = 0; a < temp_numShift.length; a++){
									if(shiftRes.numShiftCV[c1][v1] == temp_numShift[a] && flowXtemp[i][j][c1]==1 &&  numVehRemNumShift > 0){
										//System.out.print(temp_numShift[a] + " ");
										flowXtemp[i][j][c1] = 0;
										numVehRemNumShift = numVehRemNumShift - 1;
										temp_numShift[a]=-1;
									}
								}
							}
						}
					}
				}
			}
		}
		
	
			
			
			//System.out.println();
			//System.out.println();
		
	}
	

	void destroyOperator6_shift(double prcShiftCost, double prcRouteCost, double prcNumShift, ShiftingResult shiftRes, Cargo[] Cargoes, int exitI, int exitJ){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		setNewVehiclePlacement(Cargoes);
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int totalVehOnDeck = 0;
		
		for(int c = 0; c < nCargoes; c++){
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))
				totalVehOnDeck = totalVehOnDeck + Cargoes[c].NumberVehicle;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//     Remove a given percentage of the vehicles that have the highest shifting cost for a single vehicle
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		int numVehRemShift = (int) Math.ceil(totalVehOnDeck*prcShiftCost);
		
		List<Double> sortedListVehicleShiftCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleShiftCost.add(shiftRes.shiftCostCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.shiftCostCV[c][v] + "  ");
				}
			}
		}

		//System.out.println();
		//System.out.println(sortedListVehicleShiftCost.size() + " HEIHEIEHEIE");
		Collections.sort(sortedListVehicleShiftCost);
		Collections.reverse(sortedListVehicleShiftCost);
		
		double[] temp_shiftCost = new double[Math.min(numVehRemShift, sortedListVehicleShiftCost.size())];
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			temp_shiftCost[i] = sortedListVehicleShiftCost.get(i);
		}
		
		/*
		
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleShiftCost.size(); i++){
			System.out.print(sortedListVehicleShiftCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int a = 0; a < temp_shiftCost.length; a++){
							if(shiftRes.shiftCostCV[c1][v1] == temp_shiftCost[a] && flowXtemp[i][j][c1]==1 &&  numVehRemShift > 0){
								flowXtemp[i][j][c1] = 0;
								numVehRemShift = numVehRemShift - 1;
								temp_shiftCost[a]=-1;
								//System.out.print(temp_shiftCost[a] + " ");
							}
						}
					}
				}
			}
		}
									
									
		

		//System.out.println();
		//System.out.println();
			
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//	
		//                  Remove vehicles with the highest shifting cost for a given route. 
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
			
		int numVehRemRoute = (int) Math.ceil(totalVehOnDeck*prcRouteCost);
		
		List<Double> sortedListVehicleRouteCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				for(int p = 0; p < 2; p++){
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.routeCost[c][v][p] > 0.5){
						sortedListVehicleRouteCost.add(shiftRes.routeCost[c][v][p]);	
						//System.out.println(shiftRes.routeCost[c][v][p] + " ");
					}
				}
			}
		}
		//System.out.println();
		//System.out.println();
		//System.out.println(sortedListVehicleRouteCost.size() + " OKOK");
		Collections.sort(sortedListVehicleRouteCost);
		Collections.reverse(sortedListVehicleRouteCost);
		
		double[] temp_RouteCost = new double[Math.min(numVehRemRoute, sortedListVehicleRouteCost.size())];
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			temp_RouteCost[i] = sortedListVehicleRouteCost.get(i);
		}
	
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		
		System.out.println();
		for(int i = 0; i < sortedListVehicleRouteCost.size(); i++){
			System.out.print(sortedListVehicleRouteCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int p = 0; p < 2; p++){
							for(int a = 0; a < temp_RouteCost.length; a++){
								if(shiftRes.routeCost[c1][v1][p] == temp_RouteCost[a] && flowXtemp[i][j][c1]==1 && numVehRemRoute > 0){
									//System.out.print(temp_RouteCost[a] + " ");
									flowXtemp[i][j][c1] = 0;
									numVehRemRoute = numVehRemRoute - 1;
									temp_RouteCost[a]=-1;
								}
							}
						}
					}
				}
			}
		}
											
			
		
		
		//System.out.println();
		//System.out.println();
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//																												//
		//                  Remove vehicles with the highest number of other vehicles that move through it				// 
		//																												//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
		
		int numVehRemNumShift = (int) Math.ceil(totalVehOnDeck*prcNumShift);
		
		List<Double> sortedListVehicleNumShift = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleNumShift.add(shiftRes.numShiftCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.numShiftCV[c][v] + "   ");
				}
			}
		}

		Collections.sort(sortedListVehicleNumShift);
		Collections.reverse(sortedListVehicleNumShift);
		
		double[] temp_numShift = new double[Math.min(numVehRemNumShift, sortedListVehicleNumShift.size())];
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			temp_numShift[i] = sortedListVehicleNumShift.get(i);
		}
		
		
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleNumShift.size(); i++){
			System.out.print(sortedListVehicleNumShift.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int q = 0; q < nPorts; q++){
							if (vehiclePlacements[i][j].isOnShipInPort[q] == 1){
								for(int a = 0; a < temp_numShift.length; a++){
									if(shiftRes.numShiftCV[c1][v1] == temp_numShift[a] && flowXtemp[i][j][c1]==1 &&  numVehRemNumShift > 0){
										//System.out.print(temp_numShift[a] + " ");
										flowXtemp[i][j][c1] = 0;
										numVehRemNumShift = numVehRemNumShift - 1;
										temp_numShift[a]=-1;
									}
								}
							}
						}
					}
				}
			}
		}
		
		int sumArea = 0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1 && flowXtemp[i][j][c]==0){
						sumArea = sumArea + Cargoes[c].Area;
					}
				}
			}
		}
		
		int i_start = nRows-exitI;
		int i_end = nRows-exitI+1;
		int j_start = nColumns-exitJ;
		int j_end = nColumns-exitJ+1;
		int removedArea = 0;
		if (sumArea>0){	
			while (removedArea<1.5*sumArea){
				if (i_start<0){
					i_start = 0;
				}
				if (i_end>nRows){
					i_end = nRows;
				}
				if (j_start<0){
					j_start = 0;
				}
				if (j_end>nColumns){
					j_end = nColumns;
				}
				for(int i = i_start; i < i_end; i++){
					for(int j = j_start; j < j_end; j++){
						for(int c = 0; c < nCargoes; c++){
							if(flowXtemp[i][j][c]==1){
								flowXtemp[i][j][c]=0;
								removedArea = removedArea + Cargoes[c].Area;
							}
						}
					}
				}
				i_start--;
				i_end++;
				j_start--;
				j_end++;
				if (i_start<0){
					i_start = 0;
				}
				if (i_end>nRows){
					i_end = nRows;
				}
				if (j_start<0){
					j_start = 0;
				}
				if (j_end>nColumns){
					j_end = nColumns;
				}
				if (i_start == 0 && i_end == nRows && j_start == 0 && j_end == nColumns){
					removedArea = 2*sumArea;
				}
			}
		}
	
			
			
			//System.out.println();
			//System.out.println();
		
	}
	

	void destroyOperator6_route(double prcShiftCost, double prcRouteCost, double prcNumShift, ShiftingResult shiftRes, Cargo[] Cargoes, int exitI, int exitJ){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		setNewVehiclePlacement(Cargoes);
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int totalVehOnDeck = 0;
		
		for(int c = 0; c < nCargoes; c++){
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))
				totalVehOnDeck = totalVehOnDeck + Cargoes[c].NumberVehicle;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//     Remove a given percentage of the vehicles that have the highest shifting cost for a single vehicle
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		int numVehRemShift = (int) Math.ceil(totalVehOnDeck*prcShiftCost);
		
		List<Double> sortedListVehicleShiftCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleShiftCost.add(shiftRes.shiftCostCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.shiftCostCV[c][v] + "  ");
				}
			}
		}

		//System.out.println();
		//System.out.println(sortedListVehicleShiftCost.size() + " HEIHEIEHEIE");
		Collections.sort(sortedListVehicleShiftCost);
		Collections.reverse(sortedListVehicleShiftCost);
		
		double[] temp_shiftCost = new double[Math.min(numVehRemShift, sortedListVehicleShiftCost.size())];
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			temp_shiftCost[i] = sortedListVehicleShiftCost.get(i);
		}
		
		/*
		
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleShiftCost.size(); i++){
			System.out.print(sortedListVehicleShiftCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int a = 0; a < temp_shiftCost.length; a++){
							if(shiftRes.shiftCostCV[c1][v1] == temp_shiftCost[a] && flowXtemp[i][j][c1]==1 &&  numVehRemShift > 0){
								flowXtemp[i][j][c1] = 0;
								numVehRemShift = numVehRemShift - 1;
								temp_shiftCost[a]=-1;
								//System.out.print(temp_shiftCost[a] + " ");
							}
						}
					}
				}
			}
		}
									
									
		

		//System.out.println();
		//System.out.println();
			
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//	
		//                  Remove vehicles with the highest shifting cost for a given route. 
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
			
		int numVehRemRoute = (int) Math.ceil(totalVehOnDeck*prcRouteCost);
		
		List<Double> sortedListVehicleRouteCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				for(int p = 0; p < 2; p++){
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.routeCost[c][v][p] > 0.5){
						sortedListVehicleRouteCost.add(shiftRes.routeCost[c][v][p]);	
						//System.out.println(shiftRes.routeCost[c][v][p] + " ");
					}
				}
			}
		}
		//System.out.println();
		//System.out.println();
		//System.out.println(sortedListVehicleRouteCost.size() + " OKOK");
		Collections.sort(sortedListVehicleRouteCost);
		Collections.reverse(sortedListVehicleRouteCost);
		
		double[] temp_RouteCost = new double[Math.min(numVehRemRoute, sortedListVehicleRouteCost.size())];
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			temp_RouteCost[i] = sortedListVehicleRouteCost.get(i);
		}
	
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		
		System.out.println();
		for(int i = 0; i < sortedListVehicleRouteCost.size(); i++){
			System.out.print(sortedListVehicleRouteCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int p = 0; p < 2; p++){
							for(int a = 0; a < temp_RouteCost.length; a++){
								if(shiftRes.routeCost[c1][v1][p] == temp_RouteCost[a] && flowXtemp[i][j][c1]==1 && numVehRemRoute > 0){
									//System.out.print(temp_RouteCost[a] + " ");
									flowXtemp[i][j][c1] = 0;
									numVehRemRoute = numVehRemRoute - 1;
									temp_RouteCost[a]=-1;
								}
							}
						}
					}
				}
			}
		}
											
			
		
		
		//System.out.println();
		//System.out.println();
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//																												//
		//                  Remove vehicles with the highest number of other vehicles that move through it				// 
		//																												//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
		
		int numVehRemNumShift = (int) Math.ceil(totalVehOnDeck*prcNumShift);
		
		List<Double> sortedListVehicleNumShift = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleNumShift.add(shiftRes.numShiftCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.numShiftCV[c][v] + "   ");
				}
			}
		}

		Collections.sort(sortedListVehicleNumShift);
		Collections.reverse(sortedListVehicleNumShift);
		
		double[] temp_numShift = new double[Math.min(numVehRemNumShift, sortedListVehicleNumShift.size())];
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			temp_numShift[i] = sortedListVehicleNumShift.get(i);
		}
		
		
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleNumShift.size(); i++){
			System.out.print(sortedListVehicleNumShift.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (vehiclePlacements[i][j].empty==0){
					int v1 = vehiclePlacements[i][j].vehicle;
					int c1 = vehiclePlacements[i][j].cargo;
					if (flowXtemp[i][j][c1]==1){
						for(int q = 0; q < nPorts; q++){
							if (vehiclePlacements[i][j].isOnShipInPort[q] == 1){
								for(int a = 0; a < temp_numShift.length; a++){
									if(shiftRes.numShiftCV[c1][v1] == temp_numShift[a] && flowXtemp[i][j][c1]==1 &&  numVehRemNumShift > 0){
										//System.out.print(temp_numShift[a] + " ");
										flowXtemp[i][j][c1] = 0;
										numVehRemNumShift = numVehRemNumShift - 1;
										temp_numShift[a]=-1;
									}
								}
							}
						}
					}
				}
			}
		}
		
		//Sum up the area of vehicles removed.
		int sumArea = 0;
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1 && flowXtemp[i][j][c]==0){
						sumArea = sumArea + Cargoes[c].Area;
					}
				}
			}
		}
		
		int i_start = exitI;
		int i_end = exitI+1;
		int j_start = exitJ;
		int j_end = exitJ+1;
		int removedArea = 0;
		if (sumArea>0){	
			while (removedArea<1.5*sumArea){
				if (i_start<0){
					i_start = 0;
				}
				if (i_end>nRows){
					i_end = nRows;
				}
				if (j_start<0){
					j_start = 0;
				}
				if (j_end>nColumns){
					j_end = nColumns;
				}
				for(int i = i_start; i < i_end; i++){
					for(int j = j_start; j < j_end; j++){
						for(int c = 0; c < nCargoes; c++){
							if(flowXtemp[i][j][c]==1){
								flowXtemp[i][j][c]=0;
								removedArea = removedArea + Cargoes[c].Area;
							}
						}
					}
				}
				i_start--;
				i_end++;
				j_start--;
				j_end++;
				if (i_start<0){
					i_start = 0;
				}
				if (i_end>nRows){
					i_end = nRows;
				}
				if (j_start<0){
					j_start = 0;
				}
				if (j_end>nColumns){
					j_end = nColumns;
				}
				if (i_start == 0 && i_end == nRows && j_start == 0 && j_end == nColumns){
					removedArea = 2*sumArea;
				}
			}
		}
			
		
		//REMOVE area	
		/*
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1 && ((i < i_start || i > i_end) || (j < j_start || j > j_end))){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}	
		*/
			//System.out.println();
			//System.out.println();
		
	}
	


	void destroyOperator6gammel(double prcShiftCost, double prcRouteCost, double prcNumShift, ShiftingResult shiftRes, Cargo[] Cargoes){
		flowXtemp = new int [nRows][nColumns][nCargoes];
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if(flowX[i][j][c] == 1){
						flowXtemp[i][j][c]=1;
					}
				}
			}
		}
		
		int totalVehOnDeck = 0;
		
		for(int c = 0; c < nCargoes; c++){
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))
				totalVehOnDeck = totalVehOnDeck + Cargoes[c].NumberVehicle;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//     Remove a given percentage of the vehicles that have the highest shifting cost for a single vehicle
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		int numVehRemShift = (int) Math.ceil(totalVehOnDeck*prcShiftCost);
		
		List<Double> sortedListVehicleShiftCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleShiftCost.add(shiftRes.shiftCostCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.shiftCostCV[c][v] + "  ");
				}
			}
		}

		//System.out.println();
		//System.out.println(sortedListVehicleShiftCost.size() + " HEIHEIEHEIE");
		Collections.sort(sortedListVehicleShiftCost);
		Collections.reverse(sortedListVehicleShiftCost);
		
		double[] temp_shiftCost = new double[Math.min(numVehRemShift, sortedListVehicleShiftCost.size())];
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			temp_shiftCost[i] = sortedListVehicleShiftCost.get(i);
		}
		
		
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleShiftCost.size(); i++){
			System.out.print(sortedListVehicleShiftCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemShift, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_shiftCost[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
						for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
							for(int q = 0; q < nPorts; q++){
								for(int a = 0; a < temp_shiftCost.length; a++){
									if(shiftRes.shiftCostCV[c][v] == temp_shiftCost[a] && vehiclePlacement[i][j][c][v][q] == 1 && flowXtemp[i][j][c] == 1 && numVehRemShift > 0){
										//System.out.print(temp_shiftCost[a] + " ");
										flowXtemp[i][j][c] = 0;
										numVehRemShift = numVehRemShift - 1;
										temp_shiftCost[a]=-1;
									}
								}
							}
						}
					}
				}
			}
		}

		//System.out.println();
		//System.out.println();
			
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//	
		//                  Remove vehicles with the highest shifting cost for a given route. 
		//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
			
		int numVehRemRoute = (int) Math.ceil(totalVehOnDeck*prcRouteCost);
		
		List<Double> sortedListVehicleRouteCost = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				for(int p = 0; p < 2; p++){
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.routeCost[c][v][p] > 0.5){
						sortedListVehicleRouteCost.add(shiftRes.routeCost[c][v][p]);	
						//System.out.print(shiftRes.routeCost[c][v][p] + " ");
					}
				}
			}
		}
		//System.out.println();
		//System.out.println();
		//System.out.println(sortedListVehicleRouteCost.size() + " OKOK");
		Collections.sort(sortedListVehicleRouteCost);
		Collections.reverse(sortedListVehicleRouteCost);
		
		double[] temp_RouteCost = new double[Math.min(numVehRemRoute, sortedListVehicleShiftCost.size())];
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleShiftCost.size()); i++){
			temp_RouteCost[i] = sortedListVehicleRouteCost.get(i);
		}
	
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleShiftCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleRouteCost.size(); i++){
			System.out.print(sortedListVehicleRouteCost.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemRoute, sortedListVehicleRouteCost.size()); i++){
			System.out.print(temp_RouteCost[i] + " ");
		}
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
						for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
							for(int p = 0; p < 2; p++){
								for(int q = 0; q < nPorts; q++){
									for(int a = 0; a < temp_RouteCost.length; a++){
										if(shiftRes.routeCost[c][v][p] == temp_RouteCost[a] && vehiclePlacement[i][j][c][v][q] == 1 && flowXtemp[i][j][c] == 1 && numVehRemRoute > 0){
											//System.out.print(temp_RouteCost[a] + " ");
											flowXtemp[i][j][c] = 0;
											numVehRemRoute = numVehRemRoute - 1;
											temp_RouteCost[a]=-1;
										}
									}
								}
							}
						}
					}
				}
			}
		}
		//System.out.println();
		//System.out.println();
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//																												//
		//                  Remove vehicles with the highest number of other vehicles that move through it				// 
		//																												//
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
		
		int numVehRemNumShift = (int) Math.ceil(totalVehOnDeck*prcNumShift);
		
		List<Double> sortedListVehicleNumShift = new ArrayList<>();
		
		for(int c = 0; c < nCargoes; c++){
			for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && shiftRes.shiftCostCV[c][v] > 0.5){
					sortedListVehicleNumShift.add(shiftRes.numShiftCV[c][v]);	
					//System.out.print("C" + c + "V" + v + " " + shiftRes.numShiftCV[c][v] + "   ");
				}
			}
		}

		System.out.println();
		System.out.println(sortedListVehicleNumShift.size() + " Blablabla");
		Collections.sort(sortedListVehicleNumShift);
		Collections.reverse(sortedListVehicleNumShift);
		
		double[] temp_numShift = new double[Math.min(numVehRemNumShift, sortedListVehicleNumShift.size())];
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			temp_numShift[i] = sortedListVehicleNumShift.get(i);
		}
		
		
		/*
		System.out.println();
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		for(int i = 0; i < sortedListVehicleNumShift.size(); i++){
			System.out.print(sortedListVehicleNumShift.get(i) + " ");
		}
		
		System.out.println();
		
		for(int i = 0; i < Math.min(numVehRemNumShift, sortedListVehicleNumShift.size()); i++){
			System.out.print(temp_numShift[i] + " ");
		}
		System.out.println();
		*/
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
						for(int v = 0; v < Cargoes[c].NumberVehicle; v++){
							for(int q = 0; q < nPorts; q++){
								for(int a = 0; a < temp_numShift.length; a++){
									if(shiftRes.numShiftCV[c][v] == temp_numShift[a] && vehiclePlacement[i][j][c][v][q] == 1 && flowXtemp[i][j][c] == 1 && numVehRemNumShift > 0){
										//System.out.print(temp_numShift[a] + " ");
										flowXtemp[i][j][c] = 0;
										numVehRemNumShift = numVehRemNumShift - 1;
										temp_numShift[a]=-1;
									}
								}
							}
						}
					}
				}
			}
		}
			
			//System.out.println();
			//System.out.println();
		
	}
	
	
	
	void countMissingVehicles(){
		this.cargoCount = new int [nCargoesM+nCargoesO];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && flowXtemp[i][j][c] != flowX[i][j][c])
						cargoCount[c]++;
				}
			}
		}
	}
	
	void countMissingVehiclesFlowXTemp(Cargo[] Cargoes){
		this.cargoCount = new int [nCargoesM+nCargoesO];
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				for(int c = 0; c < nCargoes; c++){
					if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && flowXtemp[i][j][c]==1)
						cargoCount[c]--;
				}
			}
		}
		for(int c = 0; c < nCargoes; c++){
			if((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1))){
				cargoCount[c] = cargoCount[c] + Cargoes[c].NumberVehicle;
			}
		}
	}
	
	void isFeasible(){
		if (feasible==1){
			System.out.println("The stowage solution is feasible");
		} else {
			System.out.println("The stowage solution is INfeasible");
		}
	}


	void GRASPrepair(int [][][]possibleSquares, Cargo[] Cargoes, int searchDirParam, double randomParam, int candidateListLengthInput, int areaOrSquares){
		//Grasp repair
		int[][] takenSquares = new int[nRows][nColumns];
		int[] numberOfVehiclePlaced = new int[nCargoes];
		int tempCargo = -1;
		int sum = 0;
		int[][][]flowX = new int [nRows][nColumns][nCargoes];
		
		//Dette er lagt til
		for(int c = 0; c < nCargoes; c++){
			numberOfVehiclePlaced[c]=Cargoes[c].NumberVehicle-cargoCount[c];
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					if (flowXtemp[i][j][c] == 1){
						flowX[i][j][c]=flowXtemp[i][j][c];
						for(int ii = i; ii < i+Cargoes[c].SquaresL; ii++){
							for(int jj = j; jj <j+Cargoes[c].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
				}
			}
		}
		
		
		
		
		//RANDOM
		Random r = new Random();
		double randomChooser;
		double possibleCargoCounter = 0;
		double chooserRandomCounter = 1;
		int[] possibleCargo = new int [nCargoes];
		
		
		//SearchDir
		int[] searchDir = new int [8];
		searchDir[searchDirParam]=1;
		int squareCounter = nRows*nColumns;
		int i = 0;
		int j = 0;
		int[] arrI = new int [1];
		int[] arrJ = new int [1];
		arrI[0] = 0;
		arrJ[0] = 0;
		
		//Candidatelist
		
		int[] numPossibleSquaresCargo = new int [nCargoes];
		for (int c = 0; c < nCargoes; c++) {
			numPossibleSquaresCargo[c] = this.numPossibleSquaresCargo[c];
		}
		
		
		int candidateListMaxLength = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				candidateListMaxLength++;
			}
		}
		int candidateListLength = Math.min(candidateListLengthInput, candidateListMaxLength);
		
		int[] placedCandidate = new int [nCargoes];
		
		int[][] candidateListArea = new int [nCargoes][2];
		
		int[][] candidateListSquares = new int [nCargoes][2];
		int numberCargoesInCandidateList;
		while (squareCounter>0){
			updateIJsearchDir(arrI,arrJ,searchDir,squareCounter);
			i = arrI[0];
			j = arrJ[0];
			// Set parameters to startvalue
			for (int c = 0; c < nCargoes; c++) {
				possibleCargo[c]=0; 
				candidateListArea[c][0]=0;
				candidateListArea[c][1]=-1;
				candidateListSquares[c][0]=nRows*nColumns;
				candidateListSquares[c][1]=-1;
				placedCandidate[c]=0;
			}
			possibleCargoCounter=0;
			numberCargoesInCandidateList = 0;
			
			
			
			if (takenSquares[i][j]==0){ //Str det noe p denne plassen
				
				for (int c = 0; c < nCargoes; c++) {
					// Check possible cargoes
					if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){ //Skal cargoen plasseres?
						if (numberOfVehiclePlaced[c]<Cargoes[c].NumberVehicle){ //Er alle bilene plassert?
							//Bottom left corner
							if (searchDir[0]==1 || searchDir[1]==1){
								if (possibleSquares[i][j][c]==1){
									for (int a = i; a < i +Cargoes[c].SquaresL; a++) {
										for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Upper left corner
							if (searchDir[2]==1 || searchDir[3]==1){
								if ((i-Cargoes[c].SquaresL+1)>=0 && possibleSquares[i-Cargoes[c].SquaresL+1][j][c]==1){
									for (int a = i; a > i - Cargoes[c].SquaresL; a--) {
										for (int b = j; b < j + Cargoes[c].SquaresW; b++) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Upper right corner
							if (searchDir[4]==1 || searchDir[5]==1){
								if ((i-Cargoes[c].SquaresL+1)>=0 && (j-Cargoes[c].SquaresW+1>=0) && possibleSquares[i-Cargoes[c].SquaresL+1][j-Cargoes[c].SquaresW+1][c]==1){
									for (int a = i; a > i - Cargoes[c].SquaresL; a--) {
										for (int b = j; b > j - Cargoes[c].SquaresW; b--) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
							//Bottom right corner
							if (searchDir[6]==1 || searchDir[7]==1){
								if ((j-Cargoes[c].SquaresW+1)>=0 && possibleSquares[i][j-Cargoes[c].SquaresW+1][c]==1){
									for (int a = i; a < i + Cargoes[c].SquaresL; a++) {
										for (int b = j; b > j - Cargoes[c].SquaresW; b--) {
											sum = sum + takenSquares[a][b];
										}
									}
									if (sum==0){ //Sjekker at det ikke blir crash mellom biler.
										possibleCargo[c]=1;
										possibleCargoCounter++;
									}
								}
							}
								
							
						}
					}
					sum=0;
				} // Her slutter Cargoloopen
				// Update Canidatelist:
				
				int[] candidateListCounter = new int [candidateListLength];
				for (int c = 0; c < nCargoes; c++) {
					if (possibleCargo[c]==1 && takenSquares[i][j]==0){
						if (candidateListArea[0][0]<Cargoes[c].Area){
							candidateListArea[0][0]=Cargoes[c].Area;
							candidateListArea[0][1]=c;
							candidateListCounter[0]=1;
						}
						if (candidateListSquares[0][0]>numPossibleSquaresCargo[c]){
							candidateListSquares[0][0]=numPossibleSquaresCargo[c];
							candidateListSquares[0][1]=c;
							candidateListCounter[0]=1;
						}
					}
				}
				if (areaOrSquares==0 && candidateListCounter[0]==1){
					placedCandidate[candidateListArea[0][1]]=1;
					numberCargoesInCandidateList++;
				} 
				if (areaOrSquares==1 && candidateListCounter[0]==1){
					placedCandidate[candidateListSquares[0][1]]=1;
					numberCargoesInCandidateList++;
				}
				
				for (int n = 1; n < candidateListLength; n++) {
					for (int c = 0; c < nCargoes; c++) {
						if (possibleCargo[c]==1 && takenSquares[i][j]==0){
							if (candidateListArea[n][0]<Cargoes[c].Area && placedCandidate[c]==0 && areaOrSquares==0){
								candidateListArea[n][0]=Cargoes[c].Area;
								candidateListArea[n][1]=c;
								candidateListCounter[n]=1;
							}
							if (candidateListSquares[n][0]>numPossibleSquaresCargo[c] && placedCandidate[c]==0 && areaOrSquares==1){
								candidateListSquares[n][0]=numPossibleSquaresCargo[c];
								candidateListSquares[n][1]=c;
								candidateListCounter[n]=1;
							}
						}
					}
					if (areaOrSquares==0 && candidateListCounter[n]==1){
						placedCandidate[candidateListArea[n][1]]=1;
						numberCargoesInCandidateList++;
					} 
					if (areaOrSquares==1 && candidateListCounter[n]==1){
						placedCandidate[candidateListSquares[n][1]]=1;
						numberCargoesInCandidateList++;
					}
				}
							
				//Choose which cargo to place
				double cargoChooser;
				int chooserCargoCounter = 1;
				double numberCargoesInCandidateListDouble = numberCargoesInCandidateList;
				cargoChooser = r.nextDouble();
				for (int n = 0; n < numberCargoesInCandidateList; n++) {
					if (cargoChooser<chooserCargoCounter/numberCargoesInCandidateListDouble){
						if (areaOrSquares==0) {
							tempCargo = candidateListArea[n][1];
						} else {
							tempCargo = candidateListSquares[n][1];
						}
						break;
					} else {
						chooserCargoCounter++;
					}
				}
				
				
				//Random override if randomParam is fullfilled
				chooserRandomCounter = 1;
				randomChooser = r.nextDouble();
				if (r.nextDouble()<randomParam){
					for (int c = 0; c < nCargoes; c++) {
						if (possibleCargo[c]==1){
							if (randomChooser<chooserRandomCounter/possibleCargoCounter ){
								tempCargo = c;
								break;
							} else {
								chooserRandomCounter++;
							}
						}
					}
				}
				///
				if (tempCargo!=-1){
					numberOfVehiclePlaced[tempCargo]++;
					if (searchDir[0]==1 || searchDir[1]==1){
						flowX[i][j][tempCargo]=1;
						for (int a = i; a < i +Cargoes[tempCargo].SquaresL; a++) {
							for (int b = j; b < j + Cargoes[tempCargo].SquaresW; b++) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[2]==1 || searchDir[3]==1){
						flowX[i-Cargoes[tempCargo].SquaresL+1][j][tempCargo]=1;
						for (int a = i; a > i - Cargoes[tempCargo].SquaresL; a--) {
							for (int b = j; b < j + Cargoes[tempCargo].SquaresW; b++) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[4]==1 || searchDir[5]==1){
						flowX[i-Cargoes[tempCargo].SquaresL+1][j-Cargoes[tempCargo].SquaresW+1][tempCargo]=1;
						for (int a = i; a > i - Cargoes[tempCargo].SquaresL; a--) {
							for (int b = j; b > j - Cargoes[tempCargo].SquaresW; b--) {
								takenSquares[a][b]=1;
							}
						}
					}
					if (searchDir[6]==1 || searchDir[7]==1){
						flowX[i][j-Cargoes[tempCargo].SquaresW+1][tempCargo]=1;
						for (int a = i; a < i + Cargoes[tempCargo].SquaresL; a++) {
							for (int b = j; b > j - Cargoes[tempCargo].SquaresW; b--) {
								takenSquares[a][b]=1;
							}
						}
					}
				}
				
			}
			updateNumPossibleSquaresCargo(i,j,numPossibleSquaresCargo);
			tempCargo = -1;									
			squareCounter--;			
		}
		
		//Sjekker om lsning er gyldig.
		
		int counterCargoes = 0;
		int counterPlaced = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				if (numberOfVehiclePlaced[c]==Cargoes[c].NumberVehicle){
					counterPlaced++;
				}
				counterCargoes++;
			}
		}
		int tempAreaOverusage = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				tempAreaOverusage = tempAreaOverusage + (Cargoes[c].NumberVehicle-numberOfVehiclePlaced[c])*Cargoes[c].Area;
			}
		}
		
		if (counterCargoes == counterPlaced){
			feasible = 1;
			this.flowX = flowX;
			areaNotPlaced = 0;
		} else {
			feasible = 0;
			areaNotPlaced = tempAreaOverusage;
			//this.flowX = flowX;
		}
	}

	



	void NeighborsInsertion(int [][][]possibleSquares, Cargo[] Cargoes){
		//NeighborsInsertion
		int[][] takenSquares = new int[nRows][nColumns];
		int[] numberOfVehiclePlaced = new int[nCargoes];
		countMissingVehiclesFlowXTemp(Cargoes);
		
		//Dette er lagt til
		for(int c = 0; c < nCargoes; c++){
			numberOfVehiclePlaced[c]=Cargoes[c].NumberVehicle-cargoCount[c];
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					if (flowXtemp[i][j][c] == 1){
						for(int ii = i; ii < i+Cargoes[c].SquaresL; ii++){
							for(int jj = j; jj <j+Cargoes[c].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
				}
			}
		}
		
		
		int n = 0;
		int freeSpaceCounter = 0;
		int[][][]possibleInsertion = new int[nRows][nColumns][nCargoes];
		int largestCargo=0;
		int chosenCargo=0;
		int iterCounter=0;
		int changes=1;
		while (changes!=0){
			changes=0;
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					if (takenSquares[i][j]==0){
						for(int c = 0; c < nCargoes; c++){
							if(Cargoes[c].NumberVehicle > 1){
								if(i-Cargoes[c].SquaresL >= 0  && flowXtemp[i-Cargoes[c].SquaresL][j][c] == 1){
									n++;
								}
								if(i+Cargoes[c].SquaresL < nRows && flowXtemp[i+Cargoes[c].SquaresL][j][c] == 1){
									n++;
								}
								if(j-Cargoes[c].SquaresW >= 0  && flowXtemp[i][j-Cargoes[c].SquaresW][c] == 1){
									n++;
								}
								if(j+Cargoes[c].SquaresW < nColumns  && flowXtemp[i][j+Cargoes[c].SquaresW][c] == 1){
									n++;
								}
								freeSpaceCounter=0;
								iterCounter = 0;
								if(n != 0){
									//Her vet vi at den faktisk har en nabo
									//Sjekker om det er mulig  sette inn.
									for(int ii = i; ii < Math.min(i+Cargoes[c].SquaresL, nRows); ii++){
										for(int jj = j; jj <Math.min(j+Cargoes[c].SquaresW, nColumns); jj++){
											if (takenSquares[ii][jj]!=0){
												freeSpaceCounter++;
											}
											iterCounter++;
										}
									}
									if (freeSpaceCounter==0 && possibleSquares[i][j][c]==1 && numberOfVehiclePlaced[c]<Cargoes[c].NumberVehicle && iterCounter==Cargoes[c].Area){
										//cargo is possible
										possibleInsertion[i][j][c]=1;
									}
								}
								n = 0;
							}
						}
						
						largestCargo=0;
						for(int cc = 0; cc < nCargoes; cc++){
							if (Cargoes[cc].Area>largestCargo && possibleInsertion[i][j][cc]==1){
								largestCargo=Cargoes[cc].Area;
								chosenCargo = cc;
							}
						}
						if (largestCargo>0){
							flowXtemp[i][j][chosenCargo]=1;
							changes=1;
							numberOfVehiclePlaced[chosenCargo]++;
							for(int ii = i; ii < Math.min(i+Cargoes[chosenCargo].SquaresL, nRows); ii++){
								for(int jj = j; jj <Math.min(j+Cargoes[chosenCargo].SquaresW, nColumns); jj++){
									takenSquares[ii][jj]=1;
								}
							}
						} 
					}
				}
			}
		}
	}
		
		

	void repairOperator3(StowagePlan stowagePlan, int [][][]possibleSquares, Cargo[] Cargoes,int exitI, int exitJ, int[][] UnusableSpace, int[][] ExitSquares){
		//PlacementRepair
		countMissingVehiclesFlowXTemp(Cargoes);
		//long startTime = System.currentTimeMillis();
		//System.out.println("Tid: " + (System.currentTimeMillis()-startTime));
		
		int[][] takenSquares = new int[nRows][nColumns];
		int[] numberOfVehiclePlaced = new int[nCargoes];
		int[][][]flowX = new int [nRows][nColumns][nCargoes];
		
		//Dette er lagt til
		for(int c = 0; c < nCargoes; c++){
			numberOfVehiclePlaced[c]=Cargoes[c].NumberVehicle-cargoCount[c];
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					if (flowXtemp[i][j][c] == 1){
						for(int ii = i; ii < i+Cargoes[c].SquaresL; ii++){
							for(int jj = j; jj <j+Cargoes[c].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
					if (UnusableSpace[i][j]==1 || ExitSquares[i][j]==1){
						takenSquares[i][j]=1;
					}
				}
			}
		}
		
		
		Comparator<Placement> comparator = new SquareCostComparator();
		PriorityQueue<Placement> pq1 = new PriorityQueue<Placement>(comparator);
		
		Placement[][] Placements = new Placement[nRows][nColumns]; 

		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (takenSquares[i][j]==0 && UnusableSpace[i][j]==0 && ExitSquares[i][j]==0){
					Placements[i][j] = new Placement(i,j);
					Placements[i][j].SquareCost = squareCost[i][j];
					pq1.offer(Placements[i][j]);
				}
			}
		}
		
		
		//int counterif=0;
		//int counterall=0;
		//System.out.println("fr lkkestart: " + (System.currentTimeMillis()-startTime));
		int[][][] possibleCargoInsertionDirection = new int[nRows][nColumns][nCargoes];
		while (pq1.isEmpty()==false){
			
			//counterall++;
			int temp_i=0;
			int temp_j=0;
			int freeSpaceCounter=0;
			int price=0;
			int bestPrice;
			
			temp_i=pq1.peek().i;
			temp_j=pq1.poll().j;
			
			
			
			//System.out.println("BESTI: " +temp_i + "  BESTJ: " +temp_j );
			if (takenSquares[temp_i][temp_j]==0){
				//counterif++;
				//System.out.println("Tid1: " + (System.currentTimeMillis()-startTime));
				
				for (int c = 0; c < nCargoes; c++) {
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && cargoCount[c]!=0){
						bestPrice = nRows*nColumns*(nRows+nColumns);
						for(int n = 0; n < 4; n++){
							if(n == 0 && temp_i+Cargoes[c].SquaresL < nRows-1 && temp_j+Cargoes[c].SquaresW < nColumns-1){ //opphyre
								freeSpaceCounter=0;
								price=0;
								for(int ii = temp_i; ii < temp_i+Cargoes[c].SquaresL; ii++){
									for(int jj = temp_j; jj <temp_j+Cargoes[c].SquaresW; jj++){
										if (takenSquares[ii][jj]!=0){
											freeSpaceCounter++;
											price = price + squareCost[ii][jj];
										}
									}
								}
								
								if (freeSpaceCounter == 0){
									//System.out.println("opphyre mulig");
									if (price<bestPrice){
										bestPrice = price;
										possibleCargoInsertionDirection[temp_i][temp_j][c]=n+10;
									}
								}
							}
							if(n == 1 && temp_i-Cargoes[c].SquaresL > 0 && temp_j+Cargoes[c].SquaresW < nColumns-1){ //nedhyre
								freeSpaceCounter=0;
								price=0;
								for(int ii = temp_i; ii > temp_i-Cargoes[c].SquaresL; ii--){
									for(int jj = temp_j; jj <temp_j+Cargoes[c].SquaresW; jj++){
										if (takenSquares[ii][jj]!=0){
											freeSpaceCounter++;
											price = price + squareCost[ii][jj];
										}
									}
								}
								if (freeSpaceCounter == 0){
									//System.out.println("nedhyre mulig");
									if (price<bestPrice){
										bestPrice = price;
										possibleCargoInsertionDirection[temp_i][temp_j][c]=n+10;
									}
								}
							}
							if(n == 2 && temp_i-Cargoes[c].SquaresL > 0 && temp_j-Cargoes[c].SquaresW > 0){ //nedvenstre
								freeSpaceCounter=0;
								price=0;
								for(int ii = temp_i; ii > temp_i-Cargoes[c].SquaresL; ii--){
									for(int jj = temp_j; jj >temp_j-Cargoes[c].SquaresW; jj--){
										if (takenSquares[ii][jj]!=0){
											freeSpaceCounter++;
											price = price + squareCost[ii][jj];
										}
									}
								}
								if (freeSpaceCounter == 0){
									//System.out.println("nedvenstre mulig");
									if (price<bestPrice){
										bestPrice = price;
										possibleCargoInsertionDirection[temp_i][temp_j][c]=n+10;
									}
								}
							}
							if(n == 3 && temp_i+Cargoes[c].SquaresL < nRows-1 && temp_j-Cargoes[c].SquaresW > 0){ //oppvenstre
								freeSpaceCounter=0;
								price=0;
								for(int ii = temp_i; ii < temp_i+Cargoes[c].SquaresL; ii++){
									for(int jj = temp_j; jj >temp_j-Cargoes[c].SquaresW; jj--){
										if (takenSquares[ii][jj]!=0){
											freeSpaceCounter++;
											price = price + squareCost[ii][jj];
										}
									}
								}
								if (freeSpaceCounter == 0){
									//System.out.println("oppvenstre mulig");
									if (price<bestPrice){
										bestPrice = price;
										possibleCargoInsertionDirection[temp_i][temp_j][c]=n+10;
									}
								}
							}
						}
						//Ferdig med n-lkke
						
					}
				}
				
				//Ferdig med c.lkke.
				//Finner strste mulige cargo.
				
				//System.out.println("BesteDirection" + possibleCargoInsertionDirection[temp_i][temp_j][0]);
				
				int bestCargo = -1;
				int largestArea = 0;
				int nports = 0;
				for (int c = 0; c < nCargoes; c++) {
					if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
						if (possibleCargoInsertionDirection[temp_i][temp_j][c]>=10){
							if (nports<(Cargoes[c].UnloadingPort-Cargoes[c].LoadingPort)){
								nports=(Cargoes[c].UnloadingPort-Cargoes[c].LoadingPort);
								bestCargo=c;
								largestArea = Cargoes[c].Area;
							}
							if (nports==(Cargoes[c].UnloadingPort-Cargoes[c].LoadingPort)){
								if (largestArea < Cargoes[c].Area){
									nports=(Cargoes[c].UnloadingPort-Cargoes[c].LoadingPort);
									bestCargo=c;
									largestArea = Cargoes[c].Area;
								}
							}
						}
					}
				}
				//Beste cargo funnet. Direction funnet. Vi setter inn.
				if (bestCargo!=-1){
					cargoCount[bestCargo]--;
					//System.out.println("Cargo" + bestCargo);
					//System.out.println("BesteDirection" + possibleCargoInsertionDirection[temp_i][temp_j][bestCargo]);
					if(possibleCargoInsertionDirection[temp_i][temp_j][bestCargo] == 10){ //opphyre
						flowX[temp_i][temp_j][bestCargo]=1;
						for(int ii = temp_i; ii < temp_i+Cargoes[bestCargo].SquaresL; ii++){
							for(int jj = temp_j; jj <temp_j+Cargoes[bestCargo].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
					if(possibleCargoInsertionDirection[temp_i][temp_j][bestCargo] == 11){ //nedhyre
						flowX[temp_i-Cargoes[bestCargo].SquaresL+1][temp_j][bestCargo]=1;
						for(int ii = temp_i; ii > temp_i-Cargoes[bestCargo].SquaresL; ii--){
							for(int jj = temp_j; jj <temp_j+Cargoes[bestCargo].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
					if(possibleCargoInsertionDirection[temp_i][temp_j][bestCargo] == 12){ //nedvenstre
						flowX[temp_i-Cargoes[bestCargo].SquaresL+1][temp_j-Cargoes[bestCargo].SquaresW+1][bestCargo]=1;
						for(int ii = temp_i; ii > temp_i-Cargoes[bestCargo].SquaresL; ii--){
							for(int jj = temp_j; jj >temp_j-Cargoes[bestCargo].SquaresW; jj--){
								takenSquares[ii][jj]=1;
							}
						}
					}
					if(possibleCargoInsertionDirection[temp_i][temp_j][bestCargo] == 13){ //oppvenstre
						flowX[temp_i][temp_j-Cargoes[bestCargo].SquaresW+1][bestCargo]=1;
						for(int ii = temp_i; ii < temp_i+Cargoes[bestCargo].SquaresL; ii++){
							for(int jj = temp_j; jj >temp_j-Cargoes[bestCargo].SquaresW; jj--){
								takenSquares[ii][jj]=1;
							}
						}
					}
				}
				
			}
		}
		/*
		int counter = 0;
		for (int i = 0; i < flowX.length; i++) {
			System.out.print("------");
		}
		System.out.println();
		for (int i = nRows-1; i >= 0; i--) {//
			for (int j = 0; j < nColumns; j++) {
				System.out.print("|");
				
				if (counter==0){
					System.out.print(" " +squareCost[i][j]+ "  ");
				}
				counter = 0;
			}
			System.out.print("|");
			System.out.println();
			for (int i1 = 0; i1 < nRows; i1++) {
				System.out.print("------");
			}
			System.out.println();
		}
		for(int c = 0; c < nCargoes; c++){
			System.out.println(cargoCount[c]);
		}
		*/
		
		
		
		int counterCargoes = 0;
		int counterPlaced = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				if (cargoCount[c]==0){
					counterPlaced++;
				}
				counterCargoes++;
			}
		}
		
		if (counterCargoes == counterPlaced){
			feasible = 1;
			
			areaNotPlaced = 0;
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					for(int c = 0; c < nCargoes; c++){
						if (flowX[i][j][c] == 1 || flowXtemp[i][j][c]==1){
							this.flowX[i][j][c]=1;
						} else {
							this.flowX[i][j][c]=0;
						}
					}
				}
			}
			
			
		} else {
			feasible = 0;
			//this.flowX = flowX;
		}
		
		
		
	}
		
		
	void repairOperator1(StowagePlan stowagePlan, Cargo[] Cargoes, long maxRunTime, int[] InitialWeightParamSearchDir, int[] InitialWeightParamRandom, int[] InitialWeightParamCandidateListLength, int[] InitialWeightParamAreaOrSquares){
		int[] WeightParamSearchDir = new int[InitialWeightParamSearchDir.length];
		int[] WeightParamRandom = new int[InitialWeightParamRandom.length];
		int[] WeightParamCandidateListLength = new int[InitialWeightParamCandidateListLength.length]; 
		int[] WeightParamAreaOrSquares = new int[InitialWeightParamAreaOrSquares.length];
		for (int i = 0; i < InitialWeightParamSearchDir.length; i++){
			WeightParamSearchDir[i]=InitialWeightParamSearchDir[i];
		}
		for (int i = 0; i < InitialWeightParamRandom.length; i++){
			WeightParamRandom[i]=InitialWeightParamRandom[i];
		}
		for (int i = 0; i < InitialWeightParamCandidateListLength.length; i++){
			WeightParamCandidateListLength[i]=InitialWeightParamCandidateListLength[i];
		}
		for (int i = 0; i < InitialWeightParamAreaOrSquares.length; i++){
			WeightParamAreaOrSquares[i]=InitialWeightParamAreaOrSquares[i];
		}
		
		stowagePlan.countMissingVehicles();
		stowagePlan.feasible = 0;
		long startTime = System.currentTimeMillis();
		int maxIterations = 30000000;
		int deltaWeight = 10;
		
		int prevAreaNotPlaced = 0;
		int tempSum = 0;
		Random rand = new Random();
		int randomNum;
		int paramChoice;
	
		//Initial values given by testing.
		int ParamSearchDir = 5;
		double ParamRandom = 0.1;
		int ParamCandidateListLength = 1;
		int ParamAreaOrSquares = 0;
		int randomChosen = 0;
		
		
		//Calculating number of cargoes carried.
		int cargoesCarried = 0;
		for (int c = 0; c < stowagePlan.nCargoes; c++) {
			if (c<stowagePlan.nCargoesM || (c>=stowagePlan.nCargoesM && stowagePlan.OptionalSetEvaluated.CargoesCarried[c-stowagePlan.nCargoesM]==1)){
				cargoesCarried++;
			}
		}
		
		//Weight of drawing chance.
		
	
		//System.out.println("Ite Dir Ran Lis Obj Not ");
		for (int n = 0; n < maxIterations ; n++) {
			if ((System.currentTimeMillis()-startTime)>=maxRunTime){
				break;
			}
			
			//Initial sker vi gjennom alle retninger for begge objectives, med 0 random og 1 i list length. Mulig dette kan finne gunstige lsninger fr randomsearch.
			if (n == 10000000){
				for (int p = 0; p < 8 ; p++) {
					for (int o = 0; o < 2 ; o++) {
						ParamSearchDir = p;
						ParamRandom = 0;
						ParamCandidateListLength = 1;
						ParamAreaOrSquares = o;
						stowagePlan.GRASPrepair(stowagePlan.possibleSquares, Cargoes, ParamSearchDir, ParamRandom, ParamCandidateListLength, ParamAreaOrSquares);
						//System.out.println(n+ "   " + ParamSearchDir+ "   " + (ParamRandom) + " " + (ParamCandidateListLength) + "   " + ParamAreaOrSquares + "    " + stowagePlan.areaNotPlaced);
						n++;
						if (stowagePlan.areaNotPlaced == 0){
							break;
						}
					}
					if (stowagePlan.areaNotPlaced == 0){
						break;
					}
				}
				if (stowagePlan.areaNotPlaced == 0){
					break;
				}
			}
			//Hvis initiell sking ikke gikk, sier vi at lengden p listen m vre lengre enn 1.
			WeightParamCandidateListLength[0]=0;
						
			//Choose which parameter to update
			paramChoice = rand.nextInt(4);
			if (paramChoice == 0) {
				//DRAW searchdir
				tempSum=0;
				for (int l = 0; l < WeightParamSearchDir.length ; l++) {
					tempSum = tempSum + WeightParamSearchDir[l];
				}
				if (tempSum == 0){
					ParamSearchDir = rand.nextInt(8);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamSearchDir.length ; l++) {
						tempSum = tempSum + WeightParamSearchDir[l];
						if (tempSum>=randomNum){
							ParamSearchDir = l;
							break;
						}
					}
				}
	        } else if (paramChoice == 1) {
	        	tempSum=0;
				for (int l = 0; l < WeightParamRandom.length ; l++) {
					tempSum = tempSum + WeightParamRandom[l];
				}
				if (tempSum == 0){
					double l1 = rand.nextInt(WeightParamRandom.length-1);
					ParamRandom = l1/(WeightParamRandom.length-1);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamRandom.length ; l++) {
						tempSum = tempSum + WeightParamRandom[l];
						if (tempSum>=randomNum){
							double l1 = l;
							randomChosen = l;
							ParamRandom = l1/(WeightParamRandom.length-1);
							break;
						}
					}
				}
				
	        } else if (paramChoice == 2) {
	        	tempSum=0;
				for (int l = 0; l < WeightParamCandidateListLength.length ; l++) {
					tempSum = tempSum + WeightParamCandidateListLength[l];
				}
				if (tempSum == 0){
					ParamCandidateListLength = rand.nextInt(cargoesCarried)+1;
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamCandidateListLength.length ; l++) {
						tempSum = tempSum + WeightParamCandidateListLength[l];
						if (tempSum>=randomNum){
							ParamCandidateListLength = l+1;
							break;
						}
					}
				}
				
	        } else {
	        	tempSum=0;
				for (int l = 0; l < WeightParamAreaOrSquares.length ; l++) {
					tempSum = tempSum + WeightParamAreaOrSquares[l];
				}
				if (tempSum == 0){
					ParamAreaOrSquares = rand.nextInt(2);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamAreaOrSquares.length ; l++) {
						tempSum = tempSum + WeightParamAreaOrSquares[l];
						if (tempSum>=randomNum){
							ParamAreaOrSquares = l;
							break;
						}
					}
				}
	        }
			
			stowagePlan.GRASPrepair(stowagePlan.possibleSquares, Cargoes, ParamSearchDir, ParamRandom, ParamCandidateListLength, ParamAreaOrSquares);
			
			//Printing some information
			//System.out.println(n+ "   " + ParamSearchDir+ "   " + (ParamRandom) + " " + (ParamCandidateListLength) + "   " + ParamAreaOrSquares + "    " + stowagePlan.areaNotPlaced);
			
			if (stowagePlan.areaNotPlaced == 0){
				//System.out.println("VI ER FERDIG!! Area not placed = " + stowagePlan.areaNotPlaced);
				stowagePlan.feasible=1;
				break;
			} else {
				//UPDATE WEIGHTS;
				
				if (n!=0){
					if (stowagePlan.areaNotPlaced>prevAreaNotPlaced){
						if (paramChoice == 0) {
							WeightParamSearchDir[ParamSearchDir]=WeightParamSearchDir[ParamSearchDir]-deltaWeight;
							if (WeightParamSearchDir[ParamSearchDir] < 100){
								WeightParamSearchDir[ParamSearchDir]=100;
							}
							//System.out.println("Searchdir reduced");
				        } else if (paramChoice == 1) {
				        	WeightParamRandom[randomChosen]=WeightParamRandom[randomChosen]-deltaWeight;
				        	//System.out.println("Random reduced");
				        	if (WeightParamRandom[randomChosen] < 100){
				        		WeightParamRandom[randomChosen]=100;
							}
				        } else if (paramChoice == 2) {
				        	WeightParamCandidateListLength[ParamCandidateListLength-1]=WeightParamCandidateListLength[ParamCandidateListLength-1]-deltaWeight;
				        	//System.out.println("List reduced");
				        	if (WeightParamCandidateListLength[ParamCandidateListLength-1] < 100){
				        		WeightParamCandidateListLength[ParamCandidateListLength-1]=100;
							}
				        } else {
				        	WeightParamAreaOrSquares[ParamAreaOrSquares]=WeightParamAreaOrSquares[ParamAreaOrSquares]-deltaWeight;
				        	//System.out.println("Obj reduced");
				        	if (WeightParamAreaOrSquares[ParamAreaOrSquares] < 100){
				        		WeightParamAreaOrSquares[ParamAreaOrSquares]=100;
							}
				        }
					}
					if (stowagePlan.areaNotPlaced<prevAreaNotPlaced){
						if (paramChoice == 0) {
							WeightParamSearchDir[ParamSearchDir]=WeightParamSearchDir[ParamSearchDir]+deltaWeight;
							//System.out.println("Searchdir increased");
				        } else if (paramChoice == 1) {
				        	WeightParamRandom[randomChosen]=WeightParamRandom[randomChosen]+deltaWeight;
				        	//System.out.println("Random increased");
				        } else if (paramChoice == 2) {
				        	WeightParamCandidateListLength[ParamCandidateListLength-1]=WeightParamCandidateListLength[ParamCandidateListLength-1]+deltaWeight;
				        	//System.out.println("List increased");
				        } else {
				        	WeightParamAreaOrSquares[ParamAreaOrSquares]=WeightParamAreaOrSquares[ParamAreaOrSquares]+deltaWeight;
				        	//System.out.println("Obj increased");
				        }
					}
				}
				
				prevAreaNotPlaced = stowagePlan.areaNotPlaced;
			}
		}
		//printParameters(WeightParamSearchDir,WeightParamRandom,WeightParamCandidateListLength,WeightParamAreaOrSquares);
		
		
	}
	
	
	
	void repairOperator2(StowagePlan stowagePlan, Cargo[] Cargoes, long maxRunTime, int deltaWeight, int[] InitialWeightParamSearchDir, int[] InitialWeightParamRandom, int[] InitialWeightParamCandidateListLength, int[] InitialWeightParamAreaOrSquares){
		int[] WeightParamSearchDir = new int[InitialWeightParamSearchDir.length];
		int[] WeightParamRandom = new int[InitialWeightParamRandom.length];
		int[] WeightParamCandidateListLength = new int[InitialWeightParamCandidateListLength.length]; 
		int[] WeightParamAreaOrSquares = new int[InitialWeightParamAreaOrSquares.length];
		for (int i = 0; i < InitialWeightParamSearchDir.length; i++){
			WeightParamSearchDir[i]=InitialWeightParamSearchDir[i];
		}
		for (int i = 0; i < InitialWeightParamRandom.length; i++){
			WeightParamRandom[i]=InitialWeightParamRandom[i];
		}
		for (int i = 0; i < InitialWeightParamCandidateListLength.length; i++){
			WeightParamCandidateListLength[i]=InitialWeightParamCandidateListLength[i];
		}
		for (int i = 0; i < InitialWeightParamAreaOrSquares.length; i++){
			WeightParamAreaOrSquares[i]=InitialWeightParamAreaOrSquares[i];
		}
		NeighborsInsertion(possibleSquares, Cargoes);
		countMissingVehiclesFlowXTemp(Cargoes);
		stowagePlan.feasible = 0;
		
		long startTime = System.currentTimeMillis();
		int maxIterations = 30000000;
		
		
		int prevAreaNotPlaced = 0;
		int tempSum = 0;
		Random rand = new Random();
		int randomNum;
		int paramChoice;
	
		//Initial values given by testing.
		int ParamSearchDir = 5;
		double ParamRandom = 0.1;
		int ParamCandidateListLength = 1;
		int ParamAreaOrSquares = 0;
		int randomChosen = 0;
		
		
		//Calculating number of cargoes carried.
		int cargoesCarried = 0;
		for (int c = 0; c < stowagePlan.nCargoes; c++) {
			if (c<stowagePlan.nCargoesM || (c>=stowagePlan.nCargoesM && stowagePlan.OptionalSetEvaluated.CargoesCarried[c-stowagePlan.nCargoesM]==1)){
				cargoesCarried++;
			}
		}
		
		//Weight of drawing chance.
		
	
		//System.out.println("Ite Dir Ran Lis Obj Not ");
		for (int n = 0; n < maxIterations ; n++) {
			if ((System.currentTimeMillis()-startTime)>=maxRunTime){
				break;
			}
			
			//Initial sker vi gjennom alle retninger for begge objectives, med 0 random og 1 i list length. Mulig dette kan finne gunstige lsninger fr randomsearch.
			if (n == 10000000){
				for (int p = 0; p < 8 ; p++) {
					for (int o = 0; o < 2 ; o++) {
						ParamSearchDir = p;
						ParamRandom = 0;
						ParamCandidateListLength = 1;
						ParamAreaOrSquares = o;
						stowagePlan.GRASPrepair(stowagePlan.possibleSquares, Cargoes, ParamSearchDir, ParamRandom, ParamCandidateListLength, ParamAreaOrSquares);
						//System.out.println(n+ "   " + ParamSearchDir+ "   " + (ParamRandom) + " " + (ParamCandidateListLength) + "   " + ParamAreaOrSquares + "    " + stowagePlan.areaNotPlaced);
						n++;
						if (stowagePlan.areaNotPlaced == 0){
							break;
						}
					}
					if (stowagePlan.areaNotPlaced == 0){
						break;
					}
				}
				if (stowagePlan.areaNotPlaced == 0){
					break;
				}
			}
			//Hvis initiell sking ikke gikk, sier vi at lengden p listen m vre lengre enn 1.
			WeightParamCandidateListLength[0]=0;
						
			//Choose which parameter to update
			paramChoice = rand.nextInt(4);
			if (paramChoice == 0) {
				//DRAW searchdir
				tempSum=0;
				for (int l = 0; l < WeightParamSearchDir.length ; l++) {
					tempSum = tempSum + WeightParamSearchDir[l];
				}
				if (tempSum == 0){
					ParamSearchDir = rand.nextInt(8);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamSearchDir.length ; l++) {
						tempSum = tempSum + WeightParamSearchDir[l];
						if (tempSum>=randomNum){
							ParamSearchDir = l;
							break;
						}
					}
				}
	        } else if (paramChoice == 1) {
	        	tempSum=0;
				for (int l = 0; l < WeightParamRandom.length ; l++) {
					tempSum = tempSum + WeightParamRandom[l];
				}
				if (tempSum == 0){
					double l1 = rand.nextInt(WeightParamRandom.length-1);
					ParamRandom = l1/(WeightParamRandom.length-1);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamRandom.length ; l++) {
						tempSum = tempSum + WeightParamRandom[l];
						if (tempSum>=randomNum){
							double l1 = l;
							randomChosen = l;
							ParamRandom = l1/(WeightParamRandom.length-1);
							break;
						}
					}
				}
				
	        } else if (paramChoice == 2) {
	        	tempSum=0;
				for (int l = 0; l < WeightParamCandidateListLength.length ; l++) {
					tempSum = tempSum + WeightParamCandidateListLength[l];
				}
				if (tempSum == 0){
					ParamCandidateListLength = rand.nextInt(cargoesCarried)+1;
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamCandidateListLength.length ; l++) {
						tempSum = tempSum + WeightParamCandidateListLength[l];
						if (tempSum>=randomNum){
							ParamCandidateListLength = l+1;
							break;
						}
					}
				}
				
	        } else {
	        	tempSum=0;
				for (int l = 0; l < WeightParamAreaOrSquares.length ; l++) {
					tempSum = tempSum + WeightParamAreaOrSquares[l];
				}
				if (tempSum == 0){
					ParamAreaOrSquares = rand.nextInt(2);
				} else {
					randomNum = rand.nextInt(tempSum) + 1;
					tempSum=0;
					for (int l = 0; l < WeightParamAreaOrSquares.length ; l++) {
						tempSum = tempSum + WeightParamAreaOrSquares[l];
						if (tempSum>=randomNum){
							ParamAreaOrSquares = l;
							break;
						}
					}
				}
	        }
			
			stowagePlan.GRASPrepair(stowagePlan.possibleSquares, Cargoes, ParamSearchDir, ParamRandom, ParamCandidateListLength, ParamAreaOrSquares);
			
			//Printing some information
			//System.out.println(n+ "   " + ParamSearchDir+ "   " + (ParamRandom) + " " + (ParamCandidateListLength) + "   " + ParamAreaOrSquares + "    " + stowagePlan.areaNotPlaced);
			
			if (stowagePlan.areaNotPlaced == 0){
				//System.out.println("VI ER FERDIG!! Area not placed = " + stowagePlan.areaNotPlaced);
				stowagePlan.feasible=1;
				break;
			} else {
				//UPDATE WEIGHTS;
				
				if (n!=0){
					if (stowagePlan.areaNotPlaced>prevAreaNotPlaced){
						if (paramChoice == 0) {
							WeightParamSearchDir[ParamSearchDir]=WeightParamSearchDir[ParamSearchDir]-deltaWeight;
							if (WeightParamSearchDir[ParamSearchDir] < 100){
								WeightParamSearchDir[ParamSearchDir]=100;
							}
							//System.out.println("Searchdir reduced");
				        } else if (paramChoice == 1) {
				        	WeightParamRandom[randomChosen]=WeightParamRandom[randomChosen]-deltaWeight;
				        	//System.out.println("Random reduced");
				        	if (WeightParamRandom[randomChosen] < 100){
				        		WeightParamRandom[randomChosen]=100;
							}
				        } else if (paramChoice == 2) {
				        	WeightParamCandidateListLength[ParamCandidateListLength-1]=WeightParamCandidateListLength[ParamCandidateListLength-1]-deltaWeight;
				        	//System.out.println("List reduced");
				        	if (WeightParamCandidateListLength[ParamCandidateListLength-1] < 100){
				        		WeightParamCandidateListLength[ParamCandidateListLength-1]=100;
							}
				        } else {
				        	WeightParamAreaOrSquares[ParamAreaOrSquares]=WeightParamAreaOrSquares[ParamAreaOrSquares]-deltaWeight;
				        	//System.out.println("Obj reduced");
				        	if (WeightParamAreaOrSquares[ParamAreaOrSquares] < 100){
				        		WeightParamAreaOrSquares[ParamAreaOrSquares]=100;
							}
				        }
					}
					if (stowagePlan.areaNotPlaced<prevAreaNotPlaced){
						if (paramChoice == 0) {
							WeightParamSearchDir[ParamSearchDir]=WeightParamSearchDir[ParamSearchDir]+deltaWeight;
							//System.out.println("Searchdir increased");
				        } else if (paramChoice == 1) {
				        	WeightParamRandom[randomChosen]=WeightParamRandom[randomChosen]+deltaWeight;
				        	//System.out.println("Random increased");
				        } else if (paramChoice == 2) {
				        	WeightParamCandidateListLength[ParamCandidateListLength-1]=WeightParamCandidateListLength[ParamCandidateListLength-1]+deltaWeight;
				        	//System.out.println("List increased");
				        } else {
				        	WeightParamAreaOrSquares[ParamAreaOrSquares]=WeightParamAreaOrSquares[ParamAreaOrSquares]+deltaWeight;
				        	//System.out.println("Obj increased");
				        }
					}
				}
				
				prevAreaNotPlaced = stowagePlan.areaNotPlaced;
			}
		}
		//printParameters(WeightParamSearchDir,WeightParamRandom,WeightParamCandidateListLength,WeightParamAreaOrSquares);
		
		
	}


	

	void repairOperator4(StowagePlan stowagePlan, int [][][]possibleSquares, Cargo[] Cargoes,int exitI, int exitJ, int[][] UnusableSpace, int[][] ExitSquares){
		//Random insertion
		countMissingVehiclesFlowXTemp(Cargoes);
		//System.out.println("Tid: " + (System.currentTimeMillis()-startTime));
		
		int[][] takenSquares = new int[nRows][nColumns];
		int[] numberOfVehiclePlaced = new int[nCargoes];
		int[][][]flowX = new int [nRows][nColumns][nCargoes];
		
		//Dette er lagt til
		for(int c = 0; c < nCargoes; c++){
			numberOfVehiclePlaced[c]=Cargoes[c].NumberVehicle-cargoCount[c];
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					if (flowXtemp[i][j][c] == 1){
						for(int ii = i; ii < i+Cargoes[c].SquaresL; ii++){
							for(int jj = j; jj <j+Cargoes[c].SquaresW; jj++){
								takenSquares[ii][jj]=1;
							}
						}
					}
					if (UnusableSpace[i][j]==1 || ExitSquares[i][j]==1){
						takenSquares[i][j]=1;
					}
					
				}
			}
		}
		
		Random randomRemoval = new Random();
		Square[][] squares = new Square[nRows][nColumns];
		List<Square> arrListSquare = new ArrayList<>();
		
		for(int i = 0; i < nRows; i++){
			for(int j = 0; j < nColumns; j++){
				if (takenSquares[i][j]==0){
					squares[i][j]=new Square(i,j);
					arrListSquare.add(squares[i][j]);
				}
			}
		}
		
		int random=0;
		int freeSpaceCounter=0;
		int bestCargo;
		int bestArea;
		int breaker = 1;
		
		
		while (breaker != 0){ // eller alle biler plassert
			if (arrListSquare.size()==0){
				break;
			}
			
			random = randomRemoval.nextInt(arrListSquare.size());
			breaker = 0;
			Square temp = arrListSquare.get(random);
			if (takenSquares[temp.i][temp.j]==0){
				bestCargo = -1;
				bestArea = 0;
				for (int c = 0; c < nCargoes; c++) {
					if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && cargoCount[c]!=0){
						freeSpaceCounter=0;
						if(temp.i+Cargoes[c].SquaresL < nRows-1 && temp.j+Cargoes[c].SquaresW < nColumns-1){ //opphyre
							for(int ii = temp.i; ii < temp.i+Cargoes[c].SquaresL; ii++){
								for(int jj = temp.j; jj <temp.j+Cargoes[c].SquaresW; jj++){
									if (takenSquares[ii][jj]!=0){
										freeSpaceCounter++;
									}
								}
							}
							if (freeSpaceCounter == 0){
								if (bestArea<Cargoes[c].Area){
									bestCargo = c;
									bestArea = Cargoes[c].Area;
								}
							}
						}
					}
				}
				//Her vet vi hvilke som er mulig  sette inn. Finn den strste.
				if (bestCargo!=-1){
					cargoCount[bestCargo]--;
					flowX[temp.i][temp.j][bestCargo] = 1;
					for(int ii = temp.i; ii < temp.i+Cargoes[bestCargo].SquaresL; ii++){
						for(int jj = temp.j; jj <temp.j+Cargoes[bestCargo].SquaresW; jj++){
							takenSquares[ii][jj]=1;
						}
					}
				}
				
			}
			arrListSquare.remove(random);
			for (int c = 0; c < nCargoes; c++) {
				if ((c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)) && cargoCount[c]!=0){
					breaker++;
				}
			}
				
			
			
		}
		
		
		
		int counterCargoes = 0;
		int counterPlaced = 0;
		for (int c = 0; c < nCargoes; c++) {
			if (c<nCargoesM || (c>=nCargoesM && OptionalSetEvaluated.CargoesCarried[c-nCargoesM]==1)){
				if (cargoCount[c]==0){
					counterPlaced++;
				}
				counterCargoes++;
			}
		}
		
		if (counterCargoes == counterPlaced){
			feasible = 1;
			
			areaNotPlaced = 0;
			for(int i = 0; i < nRows; i++){
				for(int j = 0; j < nColumns; j++){
					for(int c = 0; c < nCargoes; c++){
						if (flowX[i][j][c] == 1 || flowXtemp[i][j][c]==1){
							this.flowX[i][j][c]=1;
						} else {
							this.flowX[i][j][c]=0;
						}
					}
				}
			}
			
			
		} else {
			feasible = 0;
			//this.flowX = flowX;
		}
		
		
		
	}
}
		
		